/**********************************************
/*
/*	This class is taken from Frank Luna's web site,
/*	which is a simple camera class
/*
/**********************************************/

#ifndef Camera_H
#define Camera_H

#include <d3dx9.h>

class Camera
{
public:
	// By default, the Camera starts out with its basis vectors 
	// aligned with the world space axes, and its origin positioned
	// at the world space origin.
	Camera();

	// Read only accessor functions.
	const D3DXMATRIX& view() const;
	const D3DXMATRIX& proj() const;
	const D3DXMATRIX& viewProj() const;

	const D3DXVECTOR3& right() const;
	const D3DXVECTOR3& up() const;
	const D3DXVECTOR3& look() const;

	// Read/write access to the Camera position.
	D3DXVECTOR3& pos();

	// Our implementation of D3DXMatrixLookAtLH
	void lookAt(D3DXVECTOR3& pos, D3DXVECTOR3& target, D3DXVECTOR3& up);

	// Perspective projection parameters.
	void setLens(float fov, float aspect, float nearZ, float farZ);

	// Sets the Camera speed.
	void setSpeed(float s);

	// Updates the Camera's basis vectors and origin, relative to 
	// the world space, based in user input.
	void update(float mDx, float mDy, bool fw, bool bckw, bool left, bool right);


	// Constructs the view matrix based on the Camera's basis
	// vectors and origin, relative to the world space
	void buildView();

public:
	// Save Camera related matrices.
	D3DXMATRIX mView;
	D3DXMATRIX mProj;
	D3DXMATRIX mViewProj;

	// Relative to world space.
	D3DXVECTOR3 mPosW;
	D3DXVECTOR3 mRightW;
	D3DXVECTOR3 mUpW;
	D3DXVECTOR3 mLookW;

	// Camera speed.
	float mSpeed;
};

#endif // Camera_H