#include "Core.h"
#include "Stats.h"
#include "GameStats.h"

#ifndef Core_START
#define Core_START



LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch( msg )
    {
        case WM_KEYDOWN:
        {
            switch( wParam )
            {
                case VK_ESCAPE:
                    PostQuitMessage(0);
                    break;

            }
			Keyboard::keyPressed(true);
        }
	}
	return ::DefWindowProc(hwnd, msg, wParam, lParam);
}


Core::Core(	HINSTANCE hinstance )
{
	try
	{
		m_Device = NULL;
		m_Keyboard = NULL;
		m_Mouse = NULL;
		m_SceneMgr = NULL;
		m_ParticleSystem = NULL;
		m_Hwnd = 0;
		init(hinstance);

		if(m_Device == NULL )
		{
			Tools::LOG("Core::Core @ D3D9 device cannot be created : ",true);
			throw new exception();
		}
		else
		{
			Tools::LOG("Core::Core @ D3D9 device successfully created : ");
		}

		initRenderStates();

		ParticleSystem::create( m_Device );
		m_ParticleSystem = ParticleSystem::getSingleton();

		if(m_ParticleSystem == NULL)
		{
			Tools::LOG("Core::Core @ Particle System cannot be created : ",true);
			throw new exception();
		}
		else
		{
			Tools::LOG("Core::Core @ Particle System successfully created : ");
		}

		if( &m_Window == NULL)
		{
			Tools::LOG("Core::Core @ Window cannot be created : ",true);
			throw new exception();
		}
		else
		{
			Tools::LOG("Core::Core @ Window successfully created : ");
		}


		if(m_Keyboard == NULL)
		{
			Tools::LOG("Core::Core @ Keyboard cannot be created : ",true);
			throw new exception();
		}
		else
		{
			Tools::LOG("Core::Core @ Keyboard successfully created : ");
		}

		if(m_Mouse == NULL)
		{
			Tools::LOG("Core::Core @ Mouse cannot be created : ",true);
			throw new exception();
		}
		else
		{
			Tools::LOG("Core::Core @ Mouse successfully created : ");
		}

		m_SceneMgr = new SceneManager(m_Device,m_Window,m_Mouse,m_Keyboard);

		if ( m_SceneMgr == NULL )
		{
			Tools::LOG("Core::Core @ Scene Manager cannot be created : ",true);
			throw new exception();
		}
		else
		{
			Tools::LOG("Core::Core @ Scene Manager successfully created ");
		}
		Sound::scanDirectory(string(".\\"));
		m_SpriteManager = SpriteManager::getSingleton();
		m_SpriteManager->registerSprites(m_Device,string("not-found"),string("not-found.png"),20,20,1,1);
		Stats::create(m_Device);
		GameStats::create(m_Device);
		Tools::setDevice(m_Device);
	}
	catch(exception e)
	{
		string s1(e.what());
		Tools::LOG("Core::Core @ exception : "+ s1 ,true);
		throw new exception();
	}
	catch(...)
	{
		Tools::LOG("Core::Core @ exception : ",true);
		throw new exception();
	}

}

Core::~Core(void)
{
	if (m_Keyboard)
	{
		delete m_Keyboard;
	}

	if (m_Mouse)
	{
	  delete m_Mouse;
	}

	if (IODevice::getDirectInputObject())
	{
		IODevice::getDirectInputObject()->Release();
	}
	if(m_SceneMgr)
	{
		delete m_SceneMgr;
	}
}

bool Core::coreLoop(float timeDelta)
{
	try
	{
		m_SceneMgr->run();
	}
	catch(exception e)
	{
		string s1(e.what());
		Tools::LOG("Core::coreLoop @ exception " + s1, true);
		throw new exception();
	}
	catch(...)
	{
		Tools::LOG("Core::coreLoop @ exception ");
		throw new exception();
	}
	return true;
}

void Core::go()
{	
	enterMsgLoop( );
}
void Core::init(HINSTANCE hinstance)
{
	if ( !initD3D9(hinstance) )
	{
		Tools::LOG( "Core::init @ exception " );
		throw exception();
	}
	
	initKeyboard();
	initMouse();

}
bool Core::initD3D9(HINSTANCE hInstance)
{
	try
	{
		m_Window.style         = CS_HREDRAW | CS_VREDRAW;
		m_Window.lpfnWndProc   = (WNDPROC)WndProc; 
		m_Window.cbClsExtra    = 0;
		m_Window.cbWndExtra    = 0;
		m_Window.hInstance     = hInstance;
		m_Window.hIcon         = LoadIcon(0, IDI_APPLICATION);
		m_Window.hCursor       = LoadCursor(0, IDC_ARROW);
		m_Window.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
		m_Window.lpszMenuName  = 0;
		m_Window.lpszClassName = "Direct3D9App";

		DWORD width = 800,
			height = 600;
		if( !RegisterClass(&m_Window) ) 
		{
			Tools::LOG( "Core::initD3D9 @ Cannot register window ", true );
			return false;
		}
		m_Hwnd = ::CreateWindow("Direct3D9App", "Direct3D9App", 
			WS_EX_TOPMOST,
			0, 0, width, height,
			0 /*parent hwnd*/, 0 /* menu */, hInstance, 0 /*extra*/); 

		if( !m_Hwnd )
		{
			Tools::LOG( "Core::initD3D9 @ Cannot create window ", true );
			return false;
		}

		::ShowWindow(m_Hwnd, SW_SHOW);
		::UpdateWindow(m_Hwnd);

		//
		// Init D3D: 
		//

		HRESULT hr = 0;

		// Step 1: Create the IDirect3D9 object.

		IDirect3D9* d3d9 = 0;
		d3d9 = Direct3DCreate9(D3D_SDK_VERSION);

		if( !d3d9 )
		{
			Tools::LOG( "Core::initD3D9 @ Cannot create D3D9 device ", true );
			return false;
		}

		// Step 2: Check for hardware vp.

		D3DCAPS9 caps;
		d3d9->GetDeviceCaps(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &caps);

		int vp = 0;
		if( caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT )
			vp = D3DCREATE_HARDWARE_VERTEXPROCESSING;
		else
			vp = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

		// added by ldn: so's we can pick up on format height etc later
		D3DDISPLAYMODE displayMode;
		d3d9->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &displayMode);

		// Step 3: Fill out the D3DPRESENT_PARAMETERS structure.

		D3DPRESENT_PARAMETERS d3dpp;
		// clear them first
		memset(&d3dpp, 0, sizeof(D3DPRESENT_PARAMETERS));
		d3dpp.BackBufferWidth            = displayMode.Width;
		d3dpp.BackBufferHeight           = displayMode.Height;
		d3dpp.BackBufferFormat           = displayMode.Format;
		d3dpp.BackBufferCount            = 1;
		d3dpp.MultiSampleType            = D3DMULTISAMPLE_NONE;
		d3dpp.MultiSampleQuality         = 0;
		d3dpp.SwapEffect                 = D3DSWAPEFFECT_DISCARD; 
		d3dpp.hDeviceWindow              = m_Hwnd;
		d3dpp.Windowed                   = false;
		d3dpp.EnableAutoDepthStencil     = TRUE; 
		d3dpp.AutoDepthStencilFormat     = D3DFMT_D24S8;
		d3dpp.Flags                      = 0;
		d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
		d3dpp.PresentationInterval       = D3DPRESENT_INTERVAL_IMMEDIATE;


		// Step 4: Create the device.

		hr = d3d9->CreateDevice(
			D3DADAPTER_DEFAULT, // primary adapter
			D3DDEVTYPE_HAL,         // device type
			m_Hwnd,               // window associated with device
			vp,                 // vertex processing
			&d3dpp,             // present parameters
			&m_Device);            // return created device

		if( FAILED(hr) )
		{
			// try again using a 16-bit depth buffer
			d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
			
			hr = d3d9->CreateDevice(
				D3DADAPTER_DEFAULT,
				D3DDEVTYPE_HAL,
				m_Hwnd,
				vp,
				&d3dpp,
				&m_Device);

			if( FAILED(hr) )
			{
				d3d9->Release(); // done with d3d9 object
				Tools::LOG( "Core::initD3D9 @ Cannot create device", true );
				return false;
			}
		}

		d3d9->Release(); // done with d3d9 object	
		return true;

	}
	catch(exception e)
	{
		string s1(e.what());
		Tools::LOG("Core::initD3D9 @ exception : " +s1, true );
		throw new exception();
		
	}
	catch(...)
	{
		Tools::LOG("Core::initD3D9 @ exception " , true);
		throw new exception();
	}

	return false;


}


int Core::enterMsgLoop(  )
{
	MSG msg;
	::ZeroMemory(&msg, sizeof(MSG));

	static DWORD lastTime = timeGetTime(); 

	while(msg.message != WM_QUIT)
	{
		if(::PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
		{
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
		else
        {	
			DWORD currTime  = timeGetTime();
			DWORD timeDelta = (currTime - lastTime)*0.001f;

			coreLoop(timeDelta);

			lastTime = currTime;
        }
    }
    return (int)msg.wParam;
}


bool Core::initKeyboard()
{
	m_Keyboard = new Keyboard(m_Hwnd);
	return true;

}

bool Core::initMouse()
{
	m_Mouse = new Mouse(m_Hwnd);
	return true;

}


void Core::initRenderStates( void )
{
	try
	{
		Tools::pushRenderState(m_Device, D3DRS_STENCILENABLE);
		Tools::pushRenderState(m_Device,D3DRS_CULLMODE);
		Tools::pushRenderState(m_Device,D3DRS_ALPHABLENDENABLE);
		Tools::pushRenderState(m_Device,D3DRS_ZENABLE);
		Tools::pushRenderState(m_Device,D3DRS_ZWRITEENABLE);
		Tools::pushRenderState(m_Device,D3DRS_LIGHTING);

		m_Device->SetRenderState(D3DRS_STENCILENABLE, false);	
		m_Device->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
		m_Device->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
		m_Device->SetRenderState(D3DRS_ZENABLE,true);
		m_Device->SetRenderState(D3DRS_ZWRITEENABLE, true);
		m_Device->SetRenderState(D3DRS_LIGHTING, false); 
	}
	catch(...)
	{
		Tools::LOG("Core::initRenderStates @ exception : ",true);
		throw new exception();
	}
}

#elif
#define Core_START


#endif
