/**********************************************
/*
/*	This class is the root class for the game
/*	which initialises d3d9 device, mouse, keyboard 
/*	and starts the game loop.
/*
/**********************************************/


#pragma once
#define WIN32_MEAN_AND_LEAN
#include <string>

#include <d3dx9.h>
#define DIRECTINPUT_VERSION	0x0800
#include <dinput.h>
#include "SceneManager.h"
#include "SpriteManager.h"
//#include "MeshManager.h"




using namespace std;


typedef D3DXVECTOR3 Vector3 ;
typedef D3DXMATRIX Matrix3 ;

class Core
{

/*
*												*
*					Constructors 				*
*												*/
public:
	Core						( HINSTANCE hinstance );
	~Core						( void );



/*
*												*
*					Functions	 				*
*												*/

public:
	void go							( void );	// starts the message loop


protected:
	virtual void initRenderStates	( void );	// necessary render states for the game should be setted in this function

	virtual void initGame			( void ) = 0;	// pure virtual, needs to be overlaoded
	virtual void createScene		( void ) = 0;	// pure virtual, needs to be overlaoded


private:
	
	void init					( HINSTANCE hinstance );	// calls in order; initD3D9,initKeyboard and initMouse

	bool initD3D9				( HINSTANCE hinstance );	// inits directx device

	int enterMsgLoop			( void );					// message loop, calls coreLoop

	bool initKeyboard			( void );
	bool initMouse				( void );
	
	bool coreLoop				(float timeDelta);




/*
*												*
*					MEMBERS						*
*												*/
private:
	WNDCLASS					m_Window ;
	HWND						m_Hwnd ;

protected:
	IDirect3DDevice9			*m_Device ;

	Keyboard					*m_Keyboard ;
	Mouse						*m_Mouse ;

	SceneManager				*m_SceneMgr;
	ParticleSystem				*m_ParticleSystem;
	SpriteManager				*m_SpriteManager;
	//MeshManager					*m_MeshManager;




	
};
