#include "GamePlayFrameListener.h"
#include "Stats.h"
#include "GameStats.h"
GamePlayFrameListener::GamePlayFrameListener(SceneManager *p_SceneMgr, SpaceShip *p_SpaceShip, Keyboard *p_Keyboard, LandSharkGame *p_Game)
:FrameListener(10)
{
	m_SpaceShip = p_SpaceShip;
	m_SceneMgr = p_SceneMgr;
	m_LastObjectCreateTime = timeGetTime();
	m_TotalTime = 0;
	m_Difficulty = 1.0f;
	m_Keyboard = p_Keyboard;
	m_Game = p_Game;

	m_MaxDifficulty = GameVariables::getInt(string("MAX_DIFFICULTY"));
	SpriteManager::getSingleton()->registerSprites(m_SceneMgr->m_Device,string("explosion"),string("explosion.png"),50,50,1,1);
	m_MenuArrow = SpriteManager::getSingleton()->createSprite(string("explosion"),INFINITE,0,D3DXSPRITE_ALPHABLEND,D3DXVECTOR3(-350,-170,0));

	m_SceneMgr->m_Sprites.push_back(m_MenuArrow);
	if(m_MaxDifficulty > 10.0f)
		m_MaxDifficulty = 10.0f;
}
unsigned int g_Counter = 0;
GamePlayFrameListener::~GamePlayFrameListener(void)
{
}

/*
* the first part of the game runs in this function
*/
void GamePlayFrameListener::runPartOne(float p_TimeElapsed)
{
	if( timeGetTime() - m_LastObjectCreateTime > (DWORD)(700.0f / m_Difficulty) )
	{
		MultiBox *mB;
		SpeedBox *sB;
		PowerBox *pB;
		ExplosionBox *eB;

		float rndBox = Tools::rnd() ;

		if( rndBox < 5.0f / m_Difficulty)
		{
			mB = MultiBox::createMultiBox(m_SceneMgr->m_Device, 
													D3DXVECTOR3(Tools::rnd(-80.0f,80.0f),0,200));
			m_SceneMgr->addRenderableObject(mB);
			PhysicsEngine::getSingleton()->registerPhyObject(mB);
			mB->m_Speed.z = Tools::rnd(-2.0f,-1.90f)/40.0f * m_Difficulty;
		}
		else if ( rndBox < 10.0f / m_Difficulty )
		{
			sB = SpeedBox::createSpeedBox(m_SceneMgr->m_Device, 
													D3DXVECTOR3(Tools::rnd(-80.0f,80.0f),0,200));
			m_SceneMgr->addRenderableObject(sB);
			PhysicsEngine::getSingleton()->registerPhyObject(sB);
			sB->m_Speed.z = Tools::rnd(-2.0f,-1.90f)/40.0f * m_Difficulty;
		}
		else if ( rndBox < 15.0f / m_Difficulty)
		{
			pB = PowerBox::createPowerBox(m_SceneMgr->m_Device, 
													D3DXVECTOR3(Tools::rnd(-80.0f,80.0f),0,200));
			m_SceneMgr->addRenderableObject(pB);
			PhysicsEngine::getSingleton()->registerPhyObject(pB);
			pB->m_Speed.z = Tools::rnd(-2.0f,-1.90f)/40.0f * m_Difficulty;
//			pB->m_Speed.x = Tools::rnd(-.50f,.50f)/40.0f * m_Difficulty;
		}
		else 
		{
			int i;
			float power;
			int loopSZ = 1;//(int)(Tools::rnd(1,m_Difficulty)) ;
			for ( i = 0 ; i <  loopSZ; i++)
			{
				D3DXVECTOR3 posV(Tools::rnd(-290.0f,290.0f),0,200);
				eB = ExplosionBox::createExplosionBox(m_SceneMgr->m_Device, posV);
				m_SceneMgr->addRenderableObject(eB);
				PhysicsEngine::getSingleton()->registerPhyObject(eB);

				D3DXVECTOR3 vTmp;
				D3DXVec3Normalize(&vTmp ,&(m_SpaceShip->getPosition() - posV + D3DXVECTOR3(Tools::rnd(-50.0f,50.0f),0,Tools::rnd(-50.0f,50.0f))));
				eB->m_Speed = vTmp * Tools::rnd(0.20f,Tools::fastSqrt(m_Difficulty/1.2f))/25.0f;

				power = Tools::rnd(.60f,m_Difficulty/1.25f);
				eB->scale(D3DXVECTOR3(power,power,power));
				eB->m_Health *= power * power * power;
			}

		}
		m_LastObjectCreateTime = timeGetTime();
	}
}

bool firstRunOfFinal = true;
/*
* final part of the game
*/
void GamePlayFrameListener::runFinalPart(float p_TimeElapsed)
{
	if(firstRunOfFinal)
	{
		D3DXVECTOR3 posV(0,0,200);
		ExplosionBox *eB = ExplosionBox::createExplosionBox(m_SceneMgr->m_Device, posV);
		m_SceneMgr->addRenderableObject(eB);
		PhysicsEngine::getSingleton()->registerPhyObject(eB);

		eB->m_Speed.x = 0; //= vTmp /25.0f;
		eB->m_Speed.y = 0;
		eB->m_Speed.z = -0.0005f;

		float power = 60;
		eB->scale(D3DXVECTOR3(power,power,power));
		eB->m_Health *= power * power * power;
		firstRunOfFinal = false;
	}

}
int g_LastKeyPress = 0;
int	selection = 0;
bool GamePlayFrameListener::frameStarted(float p_TimeElapsed)
{
	if( GameStats::getSingleton()->m_IsMenuRunning)
	{
		m_Keyboard->update();
		if(timeGetTime()-g_LastKeyPress > 100)
		{

			if(m_Keyboard->isKeyPressed(DIK_UP))
			{
				selection--;
				if(selection < 0)
				{
					selection = 1;
				}
				

			}
			else if(m_Keyboard->isKeyPressed(DIK_DOWN))
			{
				selection++;
				if(selection > 1)
					selection = 0;
			}
			g_LastKeyPress = timeGetTime()	;
		}


		if(m_Keyboard->isKeyPressed(DIK_RETURN))
		{
			if( selection == 1 )
				exit(0);
			else if ( selection == 0 )
			{
				m_Game->startGame();
			}
			m_SceneMgr->m_Sprites.pop_back();
			return true;
		}

		if(selection == 1)
		{
			m_MenuArrow->m_RectOffset.x = -420.0f;
			m_MenuArrow->m_RectOffset.y = -300.0f;

		}
		else
		{
			m_MenuArrow->m_RectOffset.x = -350.0f;
			m_MenuArrow->m_RectOffset.y = -170.0f;
		}
	}
	else
	{


		m_TotalTime += (DWORD)p_TimeElapsed;
		if(m_TotalTime > 5000)
		{
			// difficulty changes as a sinus function
			m_Difficulty = sinf(GRAD2DEGREE( (9.0f*g_Counter) ))*(m_MaxDifficulty-1.0f);
			if ( m_Difficulty < 0 ) 
				m_Difficulty *= -1.0f;
			m_Difficulty += 1.0f;// difficulty should be minimum 1
			m_TotalTime = 0;
			g_Counter++;
		}
		if(g_Counter < 100 )
		{
			runPartOne(p_TimeElapsed);
		}
		else
		{
			runFinalPart(p_TimeElapsed);
		}
	}

	return true;
}
