

#include "GameStats.h"
#include <tchar.h>
#include <string>
#include <sstream>
#include <iostream>
using std::ostringstream;

GameStats::GameStats(LPDIRECT3DDEVICE9 p_Device)
{
	m_Device = p_Device;
	D3DXFONT_DESC fontDesc;
	fontDesc.Height          = 24;
    fontDesc.Width           = 0;
    fontDesc.Weight          = 0;
    fontDesc.MipLevels       = 1;
    fontDesc.Italic          = false;
    fontDesc.CharSet         = DEFAULT_CHARSET;
    fontDesc.OutputPrecision = OUT_DEFAULT_PRECIS;
    fontDesc.Quality         = DEFAULT_QUALITY;
    fontDesc.PitchAndFamily  = DEFAULT_PITCH | FF_DONTCARE;
    _tcscpy(fontDesc.FaceName, _T("Times New Roman"));
	m_LastRun = 0;
	m_CurrentScore = 0;
	m_DestinationScore = 0;
	m_NumOfFrags = 0;

	m_PointUpSound = Sound::getSound("pointup");
	m_PointDownSound = Sound::getSound("pointdown");
	m_PointDownSound->setVolume(1000,10000);
	m_PointUpSound->get3DBuffer()->SetMinDistance(2,DS3D_IMMEDIATE);
	m_PointUpSound->get3DBuffer()->SetMaxDistance(500,DS3D_IMMEDIATE);
	m_PointDownSound->get3DBuffer()->SetMinDistance(2,DS3D_IMMEDIATE);
	m_PointDownSound->get3DBuffer()->SetMaxDistance(500,DS3D_IMMEDIATE);
	m_NeedRefresh = true;
	m_Range = 0;
	m_BulletPower = 0;
	m_NOfBullets = 0;
	D3DXCreateFontIndirect(p_Device, &fontDesc, &m_Font);
	SpriteManager::getSingleton()->registerSprites(p_Device,string("game-panel"),string("game-panel.png"),1276,60,1,1);
	SpriteManager::getSingleton()->registerSprites(p_Device,string("numbers"),string("numbers.png"),400,40,10,1);
	m_GamePanel = SpriteManager::getSingleton()->createSprite(string("game-panel"),INFINITE,0,D3DXSPRITE_ALPHABLEND,D3DXVECTOR3(0,0,0));
	for (int i = 0 ; i < 7 ; i++)
	{
		m_Numbers[i] = SpriteManager::getSingleton()->createSprite(string("numbers"),INFINITE,0,D3DXSPRITE_ALPHABLEND,D3DXVECTOR3(-360 + 35.0f*i,-10.0f,0));
	}
	m_IsMenuRunning = true;
}

GameStats::~GameStats()
{
	if(m_Font)
		m_Font->Release();
}



void GameStats::display()
{
	if(m_IsMenuRunning)
		return;
	static std::ostringstream ost;
	int tmp = 1, i = 0; 

	if( m_NeedRefresh && timeGetTime() - m_LastRun > 100 )
	{
		int diff = m_DestinationScore - m_CurrentScore ;
		if(diff < 0)
		{
			m_CurrentScore += diff / 100 - 1;
			diff = 100 / (diff);
			m_PointDownSound->get3DBuffer()->SetVelocity(0,0,diff,DS3D_IMMEDIATE);	
			m_PointDownSound->get3DBuffer()->SetPosition(0,0,diff,DS3D_IMMEDIATE);
			m_PointDownSound->play();

		}
		else if(diff > 0)
		{
			m_CurrentScore += diff / 100 + 1;
			diff = 100 / (diff);
			m_PointUpSound->get3DBuffer()->SetVelocity(0,0,diff,DS3D_IMMEDIATE);	
			m_PointUpSound->get3DBuffer()->SetPosition(0,0,diff,DS3D_IMMEDIATE);
			m_PointUpSound->play();
		}
		else
			m_NeedRefresh = false;

		if( m_NeedRefresh )
		{
			for(i = 0; i < 7 ; i++)
			{
				m_Numbers[i]->m_CurrentFrame = (m_CurrentScore / tmp) % 10;
				m_Numbers[i]->refreshRectangle();
				tmp *= 10;
			}

		}


		ost.str("");
		ost<<"Health : " << m_Health;
		ost<<"\nLives : " << m_Lives  ;
		//if(m_NOfBullets > 0 || m_BulletPower > 0 || m_NumOfFrags > 0)
		//{
		//	ost<<"\n"<<"Bullets : "<<m_NOfBullets;
		//	ost<<"\n"<<"Fire Power : "<<m_BulletPower;
		//	ost<<"\n"<<"Range : "<<m_Range;
		//}
		m_LastRun = timeGetTime();
	}
	m_GamePanel->render(m_Device);
	for ( i = 0 ; i < 7 ; i++)
	{
		m_Numbers[i]->render(m_Device);
	}
	static RECT R = {850, 5, 0, 0};
	m_Font->DrawText(0, ost.str().c_str(), -1, &R, DT_NOCLIP, D3DCOLOR_XRGB(255,255,255));


	
}

GameStats* g_GameStatsInstance = NULL;

void GameStats::create(LPDIRECT3DDEVICE9 p_Device)
{
	if(g_GameStatsInstance == NULL)
		g_GameStatsInstance = new GameStats(p_Device);
}

GameStats* GameStats::getSingleton()
{
	return g_GameStatsInstance;
}
