#include "Keyboard.h"

Keyboard::Keyboard(HWND	&p_HWin)
:IODevice(p_HWin)
{
	m_pDirectInputDevice = NULL;        //pointer to NULL 

	if(!initDirectInput()) 	Tools::LOG("Keyboard::Keyboard @ initDirectInput failed ",true);
	else if(!initKeyboard()) Tools::LOG("Keyboard::Keyboard @ initKeyboard failed ",true); //init keyboard 
} //Keyboard

Keyboard::~Keyboard(void)
{
	if(m_pDirectInputDevice != NULL)
	{
		 m_pDirectInputDevice->Unacquire(); //unacquire device 
		 m_pDirectInputDevice->Release();   //release keyboard stuff 
		 m_pDirectInputDevice = NULL;       //pointer to NULL 
	}


} //~Keyboard

bool Keyboard::initKeyboard(void)
{
	if(FAILED(getDirectInputObject()->CreateDevice(GUID_SysKeyboard,
										&m_pDirectInputDevice,
										NULL)))
	{
		Tools::LOG("Keyboard::initKeyboard @ Cannot init keyboard ");
		return false;
	}
	if(FAILED(m_pDirectInputDevice->SetDataFormat(&c_dfDIKeyboard)))
	{
		Tools::LOG("Keyboard::initKeyboard @ Cannot SetDataFormat for keyboard  ");
		return false;
	}
	if(FAILED(m_pDirectInputDevice->SetCooperativeLevel(m_HWin,
														DISCL_FOREGROUND | 
													   DISCL_NONEXCLUSIVE)))
	{
		Tools::LOG("Keyboard::initKeyboard @ Cannot SetCooperativeLevel for keyboard ");
		return false;
	}
	if( acquireInputDevice() )
	{
		Tools::LOG("Keyboard::initKeyboard @ Cannot Acquire keyboard ");
	//	return false;	// not a big problem		
	}

	ZeroMemory(&(m_KeyBuffer[0]),sizeof(char)*256);
	ZeroMemory(&(m_LastKeyBuffer[0]),sizeof(char)*256);

	Tools::LOG("Keyboard::initKeyboard @ initKeyboard is successfully done ");


	return true;

}//initKeyboard


bool Keyboard::update(void)
{
	memcpy(m_LastKeyBuffer,m_KeyBuffer,sizeof(BYTE)*256);

	if(!m_pDirectInputDevice)
	{
		Tools::LOG("Keyboard::update @ m_pDirectInputDevice is NULL ");
		return false;
	}
	if( FAILED(m_pDirectInputDevice->GetDeviceState(sizeof(m_KeyBuffer),
												   (LPVOID)&m_KeyBuffer)))
	{

		if ( !acquireInputDevice() )	// if still the device is not back
		{
			Tools::LOG("Keyboard::update @ Cannot update keyboard, trying to init...");
			if( !initKeyboard() )		//init the device
				Tools::LOG("Keyboard::update @ Cannot init keyboard ");
	
		}

		
	}

	return true;
} //Update

bool Keyboard::isKeyPressed(int Key)
{
	if(m_KeyBuffer[Key] & 0x80)
	{
		return true;
	}

	return false;
} //KeyPressed


bool keyStatus = false;	//helper global to determine if any key is pressed

bool Keyboard::isAnyKeyPressed()
{
	return keyStatus;
}	//isAnyKeyPressed

void	Keyboard::keyPressed(bool status)
{
	keyStatus = status;
}//keyPressed
