#include "Mouse.h"

Mouse::Mouse(HWND	&p_HWin)
:IODevice(p_HWin)
{

	if(!initDirectInput()) Tools::LOG("Mouse::Mouse @ initDirectInput failed ",true);
	else if(!initMouse()) Tools::LOG("Mouse::Mouse @ initMouse failed ",true);
	

}//Mouse

Mouse::~Mouse(void)
{
	if(m_pDirectInputDevice != NULL)
	{
		 m_pDirectInputDevice->Unacquire(); //unacquire device 
		 m_pDirectInputDevice->Release();   //release keyboard stuff 
		 m_pDirectInputDevice = NULL;       //pointer to NULL 
	}
}//~Mouse

bool Mouse::initMouse(void)
{


	if(FAILED(getDirectInputObject()->CreateDevice(GUID_SysMouse,
										&m_pDirectInputDevice,
										NULL)))
	{
		Tools::LOG("Mouse::initMouse @ Cannot create mouse device ");
		return false;
	}
	if(FAILED(m_pDirectInputDevice->SetDataFormat(&c_dfDIMouse2)))
	{
		Tools::LOG("Mouse::initMouse @ Cannot SetDataFormat for mouse ");
		return false;
	}

	if(FAILED(m_pDirectInputDevice->SetCooperativeLevel(m_HWin,
													DISCL_FOREGROUND| 
													DISCL_NONEXCLUSIVE)))
	{
		Tools::LOG("Mouse::initMouse @ Cannot SetCooperativeLevel for mouse ");
		return false;
	}

	if( !acquireInputDevice() )
	{
		Tools::LOG("Mouse::initMouse @ Cannot Acquire mouse ");		
	//	return false;	// not a big problem		
	}

	m_MouseCapabilities.dwSize = sizeof(m_MouseCapabilities);
	m_pDirectInputDevice->GetCapabilities(&m_MouseCapabilities);

	if(!(m_MouseCapabilities.dwFlags & DIDC_ATTACHED))
	{
		Tools::LOG("Mouse::initMouse @ Mouse is not DIDC_ATTACHED ");
	//	return false;	// not a big problem		
	}


	//resetting all the mouse states
	ZeroMemory(&m_LastButtonDownState,sizeof(m_LastButtonDownState));
	ZeroMemory(&m_MouseButtonDownState,sizeof(m_MouseButtonDownState));
	ZeroMemory(&m_MouseButtonReleaseState,sizeof(m_MouseButtonReleaseState));
	
	Tools::LOG("Mouse::initMouse @ initMouse is successfully done ");
	::ShowCursor(FALSE);
	
	return true;
} //InitMouse

bool Mouse::update(void)
{
	memcpy(&m_LastButtonDownState, &m_MouseButtonDownState, sizeof(MouseState)); // need to store before updating

	DIMOUSESTATE2 mouseState;
	if(!m_pDirectInputDevice)
	{
		Tools::LOG("Mouse::update @ Cannot m_pDirectInputDevice is NULL ");
		return false;
	}
	if( FAILED(m_pDirectInputDevice->GetDeviceState(sizeof(mouseState),
												   (LPVOID)&mouseState)))
	{



		if ( !acquireInputDevice() )	// if still the device is not back
		{
			Tools::LOG("Mouse::update @ Cannot update mouse, trying to init... ");
			if( !initMouse() )			//init the device
				Tools::LOG("Mouse::update @ Cannot init m mouse ");
	
		}

		return false;
	}

	// constructing our own mousestate structure...	//

	ZeroMemory( &m_MouseButtonDownState, sizeof(MouseState) );		//cleaning mouse events
	ZeroMemory( &m_MouseButtonReleaseState, sizeof(MouseState) );

	// storing all the states in one byte instead of eight
	for(int i = 0 ; i < m_MouseCapabilities.dwButtons ; i++)	
	{
		m_MouseButtonDownState.buttons	|= ((BYTE)(mouseState.rgbButtons[i] != 0x00)) << i;
	}
	
	//finding the released buttons by checking previouse states
	m_MouseButtonReleaseState.buttons = (m_LastButtonDownState.buttons ^  m_MouseButtonDownState.buttons) & m_LastButtonDownState.buttons;

	m_MouseButtonDownState.x = m_MouseButtonReleaseState.x = mouseState.lX;
	m_MouseButtonDownState.y = m_MouseButtonReleaseState.y = mouseState.lY;
	m_MouseButtonDownState.z = m_MouseButtonReleaseState.z = mouseState.lZ;

	//end construction//
	return true;

}//update

bool Mouse::isButtonDown(MouseButton p_Button)
{
	return (m_MouseButtonDownState.buttons & (BYTE)p_Button);

} //isButtonDown

bool Mouse::isButtonReleased(MouseButton p_Button)
{
	return (m_MouseButtonReleaseState.buttons & (BYTE)p_Button);

} //isButtonReleased

bool Mouse::isAnyButtonDown()
{
	return (m_MouseButtonDownState.buttons != 0x00);
}//isAnyButtonDown

bool Mouse::isAnyButtonReleased()
{
	return (m_MouseButtonReleaseState.buttons != 0x00);
}//isAnyButtonReleased

bool Mouse::isMouseMoved()
{
	return !((m_MouseButtonDownState.x == 0L) && (m_MouseButtonDownState.y == 0L) && (m_MouseButtonDownState.z == 0L));
}//isMouseMoved

bool Mouse::isAnythingHappened()
{
	return (isAnyButtonDown() || isAnyButtonReleased() || isMouseMoved());
}//isAnythingHappened