#include "ParticleGroup.h"
#include "ParticleSystem.h"
/**
**	Particles needs to be grouped in this system
**	and this is the grouping class for the particles
**/

ParticleGroup::ParticleGroup		( ParticleGroupLayout *p_Layout, CollisionGroup p_CollisionGroup)
:PhysicsObject(p_CollisionGroup)
{
	m_IsDead = false;
	m_Particles = NULL;
	m_ParticleLayout = NULL;
	m_Texture = NULL;
	initParticles( p_Layout );
}

ParticleGroup::~ParticleGroup		( void )
{
}

/*
*	gets particles from the particle pool and inits particles with necessary layouts
*/
void	ParticleGroup::initParticles		( ParticleGroupLayout *p_Layout )
{
	if(p_Layout == NULL)
		return;
	m_Texture = (ParticleSystem::getSingleton()->textureMap[p_Layout->m_TextureName]);

	bool isAlreadyAllocated = false;
	if( m_Particles != NULL )
	{
		if( m_ParticleLayout->m_NumOfParticles == p_Layout->m_NumOfParticles )
		{
			isAlreadyAllocated = true;
		}

	}

	if( !isAlreadyAllocated )
	{
		if(m_Particles)
			ParticleSystem::getSingleton()->removeParticles(m_ParticleLayout->m_NumOfParticles,&m_Particles);
		ParticleSystem::getSingleton()->giveParticles(p_Layout->m_NumOfParticles,&m_Particles);
	}
	m_ParticleLayout = p_Layout;
	m_CollisionGroup = p_Layout->m_CollisionGroup;
	m_LastUpdate = 0;
	m_RemainingLife = p_Layout->m_LifeLength;
	p_Layout->initParticles( &m_Particles );

}


