#include "PhysicsEngine.h"
#include "Stats.h"
PhysicsEngine::PhysicsEngine(void)
{
}

PhysicsEngine::~PhysicsEngine(void)
{
}

/*
*	registers the physical object
*/
void	PhysicsEngine::registerPhyObject(PhysicsObject *p_PhyObj)
{
	if(p_PhyObj->m_CollisionGroup == COLLISION_GROUP_NONE)
		return;
	try
	{
		p_PhyObj->getPhysicsObjectType();
	}
	catch(exception e)
	{
		string s1(e.what());
		string s2 = "PhysicsEngine::registerPhyObject @ exception : ";
		Tools::LOG(s2 + s1);
		throw new exception();
	}
	m_PhysicsRegistery.push_back(p_PhyObj);
	
}


/*
*	deletes the physical object from storage 
*/
void PhysicsEngine::deleteFromVector( PhysicsObject *p_PhyObj)
{
	try
	{
		int i, sz = m_PhysicsRegistery.size();


		for( i  =0; i < sz ; i++ )
		{
			if( m_PhysicsRegistery[i] == p_PhyObj )
			{	
				m_PhysicsRegistery.erase(m_PhysicsRegistery.begin()+i);
				return;
			}
		}
	}
	catch(exception e)
	{
		string s1(e.what());
		string s2 = "PhysicsEngine::deleteFromVector @ exception : ";
		Tools::LOG(s2 + s1);
		throw new exception();
	}


}

/*
*	unregisters the physical object from physcs world
*/
void PhysicsEngine::unRegisterPhyObject(PhysicsObject *p_PhyObj)
{

	try
	{
		deleteFromVector(  p_PhyObj );
		p_PhyObj->m_IsUnregistered = true;
	}
	catch (...)
	{
		Tools::LOG("PhysicsEngine::unRegisterPhyObject @ exception ");
		throw new exception();
	}



}

/*
*	detecs collisions btween a particle group and a sphere
*/
void PhysicsEngine::detectParticleGroupVSSphere(ParticleGroup *p_ParticleGroup,PhysicsSphere *p_Sphere)
{
	ParticleGroupLayout	*particleGL = p_ParticleGroup->m_ParticleLayout;

	float sphereRadius = p_Sphere->getRadius();
	D3DXVECTOR3 sphereCenter = p_Sphere->getCenter();

	if(particleGL->m_CreateRectengle)// if there is a bounding box for the particle group
	{
		bool outsideFlag = particleGL->m_CreateRectengle;// if the rectangle checking enabled for the group

		// checking if the center is inside
		outsideFlag = 
			sphereCenter.z > particleGL->m_Rectangle.top 
			|| sphereCenter.z < particleGL->m_Rectangle.bottom
			|| sphereCenter.x > particleGL->m_Rectangle.left 
			|| sphereCenter.x < particleGL->m_Rectangle.right;

		if(outsideFlag)// if the center is outside, check for rectengular region intersection
		{
			RECTF &pRect = particleGL->m_Rectangle;

			outsideFlag = ((sphereCenter.z - sphereRadius) > pRect.top);
			if(outsideFlag)
			{
				return;
				outsideFlag = ((sphereCenter.z + sphereRadius) < pRect.bottom);
				if(outsideFlag)
				{
					return;
					outsideFlag = ((sphereCenter.x - sphereRadius) > pRect.right);
					if(outsideFlag)
					{
						return;
						outsideFlag = ((sphereCenter.x + sphereRadius) < pRect.left);
					}

				}
			}
			if(outsideFlag)//all tests done, object is outside of the particle rectangle
				return;	
		}
	}

	Particle **paricles = p_ParticleGroup->m_Particles;
	Particle *particle;
	float sizeOfP = particleGL->m_SizeOfParticle;

	int i, sz;
	sz = particleGL->m_NumOfParticles;

	float radSq = 
		particleGL->m_SizeOfParticle * particleGL->m_SizeOfParticle +
		sphereRadius * sphereRadius;

	D3DXVECTOR3 distV;
	float distSq;
	for( i = 0 ; i < sz ; i++)// checking for all the particles
	{
		particle = paricles[i];
		if(particle->m_IsAlive)
		{
			distV = (particle->m_Position) - sphereCenter;
			distSq = distV.x * distV.x  + distV.y * distV.y  + distV.z * distV.z ;

			if ( distSq <= radSq ){
				p_Sphere->processCollision(p_ParticleGroup);
				particle->processCollision(p_Sphere);
				if (p_Sphere->m_IsDead)
					return;
			}
		}
	}
}

/*
*	detecs collisions btween two speres
*/
void PhysicsEngine::detectSphereVSSphere(PhysicsSphere *p_Sphere1,PhysicsSphere *p_Sphere2)
{
	D3DXVECTOR3 distV = p_Sphere1->getCenter() - p_Sphere2->getCenter();
	float distSq = distV.x * distV.x  + distV.y * distV.y  + distV.z * distV.z ;
	float radSq = 
		p_Sphere1->getRadius() * p_Sphere1->getRadius() +
		p_Sphere2->getRadius() * p_Sphere2->getRadius();

	if ( distSq <= radSq ){
		p_Sphere2->processCollision(p_Sphere1);
		p_Sphere1->processCollision(p_Sphere2);
	}


}

int g_NumOfDeads = 0;
//cleans up the dead particles
void PhysicsEngine::cleanUp()
{
	int i, sz;
	i = 0 ; sz = m_PhysicsRegistery.size();
	for( ; i < sz ; i++ )
	{
		if( m_PhysicsRegistery[i]->m_IsDead )
		{
			m_PhysicsRegistery.erase(m_PhysicsRegistery.begin() + i);
			sz = m_PhysicsRegistery.size();;
			i--;
			g_NumOfDeads--;

		}
	}
	char t[10];
	sprintf(t,"%d",g_NumOfDeads);
	if( g_NumOfDeads != 0)
		Tools::LOG("PhysicsEngine::cleanUp "+string(t));
}
/*
*	looks for all the physical objects in the physics storage
*	by taking the Collision groups into account
*/
void PhysicsEngine::detectCollisions()
{
	try
	{
		int i = 0, j = 0, k = 0 ;
		int szOfVector = 0;
		

		szOfVector = m_PhysicsRegistery.size();
		PhysicsObject *pyhObject1;
		PhysicsObject *pyhObject2;
		PhysicsObject::PhysicsObjectType	type1,type2;
		g_NumOfDeads = 0;
		for(j = 0 ; j < szOfVector ;  j++ )
		{
			pyhObject1 = m_PhysicsRegistery[j];

			if(pyhObject1->m_IsDead)
			{
				g_NumOfDeads++;
				continue;
			}
			for( k = j + 1 ; k < szOfVector ; k++ )
			{
				pyhObject2 = m_PhysicsRegistery[k];
				if(pyhObject2->m_IsDead)
				{
					continue;
				}// if is dead
				else if ( pyhObject1->m_CollisionGroup == pyhObject2->m_CollisionGroup || !(pyhObject1->m_CollisionGroup & pyhObject2->m_CollisionGroup)  )
				{
					continue;
				}// if, in different collision groups or if exactly the same
				else
				{


						type1 = pyhObject1->getPhysicsObjectType();
						type2 = pyhObject2->getPhysicsObjectType();


					if( 
						type1 == PhysicsObject::PARTICLE_GROUP 
						&&
						type2 == PhysicsObject::SPHERE

						)
					{				
						detectParticleGroupVSSphere( (ParticleGroup*)(pyhObject1), (PhysicsSphere*)(pyhObject2));
					}
					else if 
					(
						type2 == PhysicsObject::PARTICLE_GROUP 
						&&
						type1 == PhysicsObject::SPHERE
					)
					{
						detectParticleGroupVSSphere((ParticleGroup*)(pyhObject2) , (PhysicsSphere*)(pyhObject1));
					}

					else if
					(					
						type1 == PhysicsObject::SPHERE 
						&&
						type2 == PhysicsObject::SPHERE
					)
					{
						detectSphereVSSphere((PhysicsSphere*)(pyhObject1) , (PhysicsSphere*)(pyhObject2));
					}
				}

			}
		}
		if ( g_NumOfDeads > 0)
			cleanUp();
	}
	catch(exception e)
	{
		string s1(e.what());
		string s2 = "PhysicsEngine::detectCollisions @ exception : ";
		Tools::LOG( s2 + s1 );
		throw new exception();
	}

}






PhysicsEngine	*g_PhyEngineInstance = NULL;

PhysicsEngine* PhysicsEngine::getSingleton()
{
	if( g_PhyEngineInstance == NULL )
	{
		g_PhyEngineInstance = new PhysicsEngine();
	}
	return g_PhyEngineInstance;
}

