#include "PresentBox.h"
#include "SpaceShip.h"
#include "GameStats.h"
//#include "MeshManager.h"

bool isFirstInit = true;
static Sound*		s_WowSound = NULL;
static Sound*		s_BooSound = NULL;
static Sound*		s_ExplosionSound = NULL;
static Sound*		s_CheerSound = NULL;
static Sound*		s_AudionceSound = NULL;
int					s_MaxSpeed;
int					s_MaxNOfBullets;
float				s_PowerOfBullets;


PresentBox::PresentBox(CollisionGroup p_CollGroup)
:RenderableObject(PRESENT_BOX),PhysicsSphere(p_CollGroup)
{
	if(isFirstInit)
	{
		s_ExplosionSound	= Sound::getSound(string("explosion3"));
		s_CheerSound		= Sound::getSound("audience_cheer_6");;
		s_AudionceSound		= Sound::getSound("audience_6");
		s_WowSound			= Sound::getSound(string("tannoy_excite_2"));
		s_BooSound			= Sound::getSound(string("audience_boo_6"));


		//string &s1 = Tools::getVariable();
		//sscanf(s1.c_str(),"%d",&s_MaxNOfBullets);
		s_MaxNOfBullets = GameVariables::getInt(string("MAX_NUM_OF_BULLETS"));

		//s1 = Tools::getVariable(string("MAX_POWER_OF_BULLETS"));
		//int tmp;
		//sscanf(s1.c_str(),"%d",&tmp);
		s_PowerOfBullets = GameVariables::getInt(string("MAX_POWER_OF_BULLETS"));;

		//s1 = Tools::getVariable(string("MAX_SPEED"));
		//sscanf(s1.c_str(),"%d",&s_MaxSpeed);
		s_MaxSpeed = GameVariables::getInt(string("MAX_SPEED"));;
		isFirstInit = false;

	}
	//registerSprites(m_Device,string("electricalsparks16_alpha"),string("electricalsparks16_alpha.jpg"),1024,1024,4,4);
	//registerSprites(m_Device,string("explosion716"),string("explosion716.jpg"),1024,1024,4,4);

	m_Sprite = NULL;
	m_RenderCount = 0;
	m_Speed.x = 0;
	m_Speed.y = 0;
	m_Speed.z = 0;
	m_LastRenderTime = timeGetTime();
	m_FirstSpriteRun = true;
	m_IsDead = false;
	
}

PresentBox::~PresentBox(void)
{
}

void PresentBox::render(LPDIRECT3DDEVICE9 p_Device)
{
	if(m_IsDead)
	{
		if(m_FirstSpriteRun)
		{
			D3DXMATRIX			tmp;

			D3DXMatrixIdentity(&tmp);
			tmp._41= worldM._41 - 20.0f;
			tmp._42= worldM._42 - 50.0f;
			tmp._43= worldM._43 + 10.0f;
			tmp._11 /= 5.0f;
			tmp._22 /= 5.0f;
			tmp._33 /= 5.0f;
			memcpy(&worldM,&tmp,sizeof(worldM));
			//worldM = tmp;
			m_FirstSpriteRun = false;
		}
		DWORD elapsedTime = timeGetTime() - m_LastRenderTime;
		if(elapsedTime > 50){
			D3DXVECTOR3 scaleV(1.05f,1.05f,1.05f);
			scale(scaleV);
			m_LastRenderTime = timeGetTime();

		}

		p_Device->SetTransform(D3DTS_WORLD,&(worldM));

//			p_Device->SetTransform(D3DTS_WORLD,&worldM);
		if(m_Sprite)
		{
			m_Sprite->render(p_Device);
		
			if( m_Sprite->m_Dead)
				this->m_CurrentState = RENDERABLE_DEAD;
		}
		else
			this->m_CurrentState = RENDERABLE_DEAD;

		PhysicsEngine::getSingleton()->unRegisterPhyObject(this);


	}
	else
	{
		DWORD elapsedTime = timeGetTime() - m_LastRenderTime;
		if(elapsedTime > 10){
			RenderableObject::rotateX(1.0f);
			RenderableObject::rotateZ(1.0f);
			RenderableObject::translate(m_Speed * elapsedTime );
			m_LastRenderTime = timeGetTime();

		}
//		Tools::LOG(this->);
		RenderableObject::render(p_Device);
	}
}
D3DXVECTOR3	PresentBox::getCenter()
{
	return getPosition();
}

float		PresentBox::getRadius()
{
	return m_BoundingSphereRadius;
}




DWORD g_LastExplosionTime = 0;
int g_NOfExplosions = 0;




void ExplosionBox::processCollision(PhysicsObject *p_PhyObject)
{
	if(p_PhyObject->getPhysicsObjectType() == PARTICLE_GROUP)
	{
		m_Health -= ((GunLayout*)((ParticleGroup*)(p_PhyObject))->m_ParticleLayout)->m_BulletPower;
		if(m_Health > 0)
		{
			ParticleSystem::getSingleton()->createParticles(new ExplosionLayout(getRadius()/5.0f + 1,&this->worldM,2000.0f,2.0f));
		}
		else
		{
			ParticleSystem::getSingleton()->createParticles(new ExplosionLayout(getRadius()/2.0f+1,&this->worldM,2000.0f,2.0f,D3DXCOLOR(0.5f,0.5f,0.8f,1.0f)));
			GameStats::getSingleton()->addFrags();
			GameStats::getSingleton()->addPoints(m_BoundingSphereRadius * 2 + 1);

			s_ExplosionSound->stop();
			s_ExplosionSound->play();
			g_NOfExplosions++;

			// number of explosion in a time interval
			if( timeGetTime() - g_LastExplosionTime < 750 )
			{
				if(g_NOfExplosions > 2)
				{
					s_WowSound->play();
					s_WowSound->stop();
					GameStats::getSingleton()->addPoints(g_NOfExplosions * 15);
				}// if
			}// if 
			else 
			{
				g_NOfExplosions = 0;
			}// else

			g_LastExplosionTime = timeGetTime();
			this->m_IsDead = true;

		}// else health...
	}// if particle group...

	else if (p_PhyObject->getPhysicsObjectType() == SPHERE)
	{
		SpaceShip *ship = dynamic_cast<SpaceShip*>(p_PhyObject);
		if( ship )
		{
			//SpaceShip *ship = ((SpaceShip*)p_PhyObject);
			ship->m_Health -= int(getRadius()*2);
			GameStats::getSingleton()->setHealth(ship->m_Health);


			GameStats::getSingleton()->addFrags(-1);
			GameStats::getSingleton()->addPoints(-m_BoundingSphereRadius );

			if(ship->m_Gun[0]->m_GunLayout->m_NumOfParticles > 1)
			{

				PrimaryGunLayout *gL = new PrimaryGunLayout(
					ship->m_Gun[0]->m_GunLayout->m_NumOfParticles-1,
					&(ship->worldM),
					ship->m_Gun[0]->m_GunLayout->m_LifeLength,
					ship->m_Gun[0]->m_GunLayout->m_SizeOfParticle,
					ship->m_Gun[0]->m_GunLayout->m_Color );
				gL->m_BulletPower = ship->m_Gun[0]->m_GunLayout->m_BulletPower;
				gL->m_SpreadFactor = ship->m_Gun[0]->m_GunLayout->m_SpreadFactor;
				gL->m_CollisionGroup = ship->m_Gun[0]->m_GunLayout->m_CollisionGroup;
				
				if(gL->m_LifeLength < 1500.0f) 
					ship->m_Gun[0]->m_GunLayout->m_LifeLength *= 1.1f;

				ship->m_Gun[0]->m_GunLayout = gL;
			}// if particles...
			if(ship->m_Gun[0]->m_GunLayout->m_BulletPower < s_PowerOfBullets )
			{
				ship->m_Gun[0]->m_GunLayout->m_BulletPower += 50.0f;
				if(ship->m_Gun[0]->m_GunLayout->m_BulletPower > s_PowerOfBullets )
					ship->m_Gun[0]->m_GunLayout->m_BulletPower = s_PowerOfBullets;

				ship->m_Gun[0]->m_GunLayout->m_Color.r = pow(s_PowerOfBullets / ship->m_Gun[0]->m_GunLayout->m_BulletPower,2);
				ship->m_Gun[0]->m_GunLayout->m_Color.g = 1.0 / ship->m_Gun[0]->m_GunLayout->m_Color.r;
				ship->m_Gun[0]->m_GunLayout->m_Color.b = sinf(GRAD2DEGREE(ship->m_Gun[0]->m_GunLayout->m_Color.r * 180.0f));

			}

			s_BooSound->stop();
			s_BooSound->play();


			ParticleSystem::getSingleton()->createParticles(new ExplosionLayout(getRadius() + 1,&this->worldM,3000.0f,2.0f));
			s_ExplosionSound->stop();
			s_ExplosionSound->play();

			GameStats::getSingleton()->setBulletPower(ship->m_Gun[0]->m_GunLayout->m_BulletPower);
			GameStats::getSingleton()->setRange(ship->m_Gun[0]->m_GunLayout->m_LifeLength);
			GameStats::getSingleton()->setNOfBullets(ship->m_Gun[0]->m_GunLayout->m_NumOfParticles);
			m_IsDead = true;

		}// if space ship
	}// else if, sphere
}

ExplosionBox::ExplosionBox()
:PresentBox(COLLISION_GROUP_E | COLLISION_GROUP_D)
{
	m_Health = 500.0f;
	m_Sprite = SpriteManager::getSingleton()->createSprite(string("explosion716"),50.0f,1);

};


ExplosionBox*	ExplosionBox::createExplosionBox(LPDIRECT3DDEVICE9 p_Device , D3DXVECTOR3 &p_Pos)
{
	ExplosionBox* pBox = new ExplosionBox();

	//MeshManager::getSingleton()->fillInRenderable( (RenderableObject*)pBox ,string("asteroid.x") );
	pBox->LoadFromXFile("asteroid.x",p_Device);
	//pBox->m_CollisionGroup = COLLISION_GROUP_E | COLLISION_GROUP_D;
	pBox->scale(D3DXVECTOR3(0.10f,0.10f,0.10f));
	pBox->teleport(p_Pos);

	return pBox;

}







PowerBox::PowerBox()
:PresentBox(COLLISION_GROUP_A)
{
	m_Health = 1000.0f;
	m_Sprite = NULL;//SpriteManager::getSingleton()->createSprite(string("electricalsparks16_alpha"),50.0f,1);

};



void PowerBox::processCollision(PhysicsObject *p_PhyObject)
{

	if (p_PhyObject->getPhysicsObjectType() == SPHERE)
	{
		SpaceShip* ship = dynamic_cast<SpaceShip*>(p_PhyObject);
		
		if(ship)
		{
			ship->m_Health += 10;
			GameStats::getSingleton()->setHealth(ship->m_Health);

			GameStats::getSingleton()->addPoints(30);
			if(ship->m_Gun[0]->m_GunLayout->m_BulletPower < s_PowerOfBullets )
			{
				ship->m_Gun[0]->m_GunLayout->m_BulletPower *= 1.10f;
				if(ship->m_Gun[0]->m_GunLayout->m_BulletPower > s_PowerOfBullets )
					ship->m_Gun[0]->m_GunLayout->m_BulletPower = s_PowerOfBullets;

				ship->m_Gun[0]->m_GunLayout->m_Color.r = pow(s_PowerOfBullets / ship->m_Gun[0]->m_GunLayout->m_BulletPower,2);
				ship->m_Gun[0]->m_GunLayout->m_Color.g = 1.0 / ship->m_Gun[0]->m_GunLayout->m_Color.r;
				ship->m_Gun[0]->m_GunLayout->m_Color.b = sinf(GRAD2DEGREE(ship->m_Gun[0]->m_GunLayout->m_Color.r * 180.0f));

			}

			ParticleSystem::getSingleton()->createParticles(new ExplosionLayout(32,&this->worldM,3000.0f,2.0f));
			m_IsDead = true;
			s_CheerSound->stop();
			s_CheerSound->play();
			GameStats::getSingleton()->setBulletPower(ship->m_Gun[0]->m_GunLayout->m_BulletPower);

		}

	}
}


PowerBox*	PowerBox::createPowerBox(LPDIRECT3DDEVICE9 p_Device , D3DXVECTOR3 &p_Pos)
{
	PowerBox* pBox = new PowerBox();
//	MeshManager::getSingleton()->fillInRenderable( (RenderableObject*)pBox ,string("power_cube.x") );

	pBox->LoadFromXFile("power_cube.x",p_Device);
	//pBox->m_CollisionGroup = COLLISION_GROUP_A;
	pBox->scale(D3DXVECTOR3(.80f,.80f,.80f));
	pBox->teleport(p_Pos);

	return pBox;

}












MultiBox::MultiBox()
:PresentBox(COLLISION_GROUP_A)
{

	m_Health = 1000.0f;
	m_Sprite = NULL;//SpriteManager::getSingleton()->createSprite(string("electricalsparks16_alpha"),50.0f,1);

};



void MultiBox::processCollision(PhysicsObject *p_PhyObject)
{

	if (p_PhyObject->getPhysicsObjectType() == SPHERE)
	{
		SpaceShip* ship = dynamic_cast<SpaceShip*>(p_PhyObject);
		
		if(ship)
		{
			GameStats::getSingleton()->addPoints(20);
			if(ship->m_Gun[0]->m_GunLayout->m_NumOfParticles < s_MaxNOfBullets - 2)
			{

				PrimaryGunLayout *gL = new PrimaryGunLayout(
					ship->m_Gun[0]->m_GunLayout->m_NumOfParticles+2,
					&(ship->worldM),
					ship->m_Gun[0]->m_GunLayout->m_LifeLength,
					ship->m_Gun[0]->m_GunLayout->m_SizeOfParticle,
					ship->m_Gun[0]->m_GunLayout->m_Color );
				gL->m_SpreadFactor = ship->m_Gun[0]->m_GunLayout->m_SpreadFactor;

				gL->m_LifeLength /= 1.1f;
				if (gL->m_LifeLength > GameVariables::getInt(string("MIN_RANGE")) )
					gL->m_LifeLength = GameVariables::getInt(string("MIN_RANGE"));


				gL->m_CollisionGroup = ship->m_Gun[0]->m_GunLayout->m_CollisionGroup;
				gL->m_BulletPower = ship->m_Gun[0]->m_GunLayout->m_BulletPower;
				gL->m_SpreadFactor = ship->m_Gun[0]->m_GunLayout->m_SpreadFactor;
				
				ship->m_Gun[0]->m_GunLayout = gL;
				
				GameStats::getSingleton()->setNOfBullets(ship->m_Gun[0]->m_GunLayout->m_NumOfParticles);
				GameStats::getSingleton()->setRange(ship->m_Gun[0]->m_GunLayout->m_LifeLength);
			}
			

			ParticleSystem::getSingleton()->createParticles(new ExplosionLayout(32,&this->worldM,3000.0f,2.0f));
			m_IsDead = true;
			s_CheerSound->stop();
			s_CheerSound->play();
		}

	}
}


MultiBox*	MultiBox::createMultiBox(LPDIRECT3DDEVICE9 p_Device , D3DXVECTOR3 &p_Pos)
{
	MultiBox* pBox = new MultiBox();

//	MeshManager::getSingleton()->fillInRenderable( (RenderableObject*)pBox ,string("multi_cube.x") );
	pBox->LoadFromXFile("multi_cube.x",p_Device);
	//pBox->m_CollisionGroup = COLLISION_GROUP_A;
	pBox->scale(D3DXVECTOR3(.80f,.80f,.80f));
	pBox->teleport(p_Pos);

	return pBox;

}
















SpeedBox::SpeedBox()
:PresentBox(COLLISION_GROUP_A)
{

	m_Health = 1000.0f;
	m_Sprite =NULL; /*SpriteManager::getSingleton()->createSprite(string("electricalsparks16_alpha"),50.0f,1)*/;

};



void SpeedBox::processCollision(PhysicsObject *p_PhyObject)
{

	if (p_PhyObject->getPhysicsObjectType() == SPHERE)
	{
		SpaceShip* ship = dynamic_cast<SpaceShip*>(p_PhyObject);
		
		if(ship)
		{
			GameStats::getSingleton()->addPoints(50);
			if(ship->m_Gun[0]->m_GunSpeed < s_MaxSpeed)
			{
				ship->m_Speed *= 1.2f;
				ship->m_Gun[0]->m_GunSpeed *= 1.3f;
				if ( ship->m_Gun[0]->m_GunSpeed > s_MaxSpeed)
					ship->m_Gun[0]->m_GunSpeed = s_MaxSpeed;
			}
			ParticleSystem::getSingleton()->createParticles(new ExplosionLayout(32,&this->worldM,3000.0f,2.0f));
			m_IsDead = true;
			s_AudionceSound->stop();
			s_AudionceSound->play();
		}

	}
}


SpeedBox*	SpeedBox::createSpeedBox(LPDIRECT3DDEVICE9 p_Device , D3DXVECTOR3 &p_Pos)
{
	SpeedBox* pBox = new SpeedBox();
//	MeshManager::getSingleton()->fillInRenderable( (RenderableObject*)pBox ,string("speed_cube.x") );

	pBox->LoadFromXFile("speed_cube.x",p_Device);
	//pBox->m_CollisionGroup = COLLISION_GROUP_A;
	pBox->scale(D3DXVECTOR3(.80f,.80f,.80f));
	pBox->teleport(p_Pos);

	return pBox;

}
















SecondGunBox::SecondGunBox()
:PresentBox(COLLISION_GROUP_A)
{

	m_Health = 1000.0f;
	m_Sprite = NULL;//SpriteManager::getSingleton()->createSprite(string("electricalsparks16_alpha"),50.0f,1);

};



void SecondGunBox::processCollision(PhysicsObject *p_PhyObject)
{

	if (p_PhyObject->getPhysicsObjectType() == SPHERE)
	{
		SpaceShip* ship = dynamic_cast<SpaceShip*>(p_PhyObject);
		
		if(ship)
		{
			if(ship->m_Gun[1] == NULL)
			{

			}
			if(ship->m_Gun[0]->m_GunSpeed < 16)
			{
				ship->m_Speed *= 1.2f;
				ship->m_Gun[0]->m_GunSpeed *= 1.3f;
				ship->m_Gun[0]->m_GunLayout->m_LifeLength *= 1.2f;
			}
			ParticleSystem::getSingleton()->createParticles(new ExplosionLayout(32,&this->worldM,3000.0f,2.0f));
			m_IsDead = true;
			s_AudionceSound->stop();
			s_AudionceSound->play();
		}

	}
}


SecondGunBox*	SecondGunBox::createSecondGunBox(LPDIRECT3DDEVICE9 p_Device , D3DXVECTOR3 &p_Pos)
{
	SecondGunBox* pBox = new SecondGunBox();
//	MeshManager::getSingleton()->fillInRenderable( (RenderableObject*)pBox ,string("speed_cube.x") );

	pBox->LoadFromXFile("speed_cube.x",p_Device);
	//pBox->m_CollisionGroup = COLLISION_GROUP_A;
	pBox->scale(D3DXVECTOR3(.80f,.80f,.80f));
	pBox->teleport(p_Pos);

	return pBox;

}
