/*
////////////////////////////////////////////////
// UMUT RIZA ERTURK / umutert@gmail.com       //
// 0703851                                    //
// university of abertay dundee               //
// 14-12-2007                                 //
// intel core duo t7100                       //
// nvidia 8400m gt                            //
// Windows Vista                              //
// M. Visual Studio 2005 v.8.05               //
// directX 9.0c                               //
////////////////////////////////////////////////
*/

#include "RenderableObject.h"
#include "Tools.h"

RenderableObject::RenderableObject( OBJTYPE t, string &p_Name)
:Renderable(p_Name)
{
	D3DXMatrixIdentity( &worldM );
	type = t;

	((this->m_Mesh)) = NULL;
	((this->m_MeshMaterials)) = NULL;
	((this->m_MeshTextures)) = NULL;
	this->m_NumMaterials = 0;
	dirA = 0;


}
RenderableObject::RenderableObject(string &p_Name)
:Renderable(p_Name)
{
	type = UNDEFINED;

	((this->m_Mesh)) = NULL;
	((this->m_MeshMaterials)) = NULL;
	((this->m_MeshTextures)) = NULL;
	this->m_NumMaterials = 0;
	dirA = 0;
	
}

RenderableObject::~RenderableObject(void)
{
	if(m_Mesh && (*m_Mesh ))
	{
		(*m_Mesh )->Release();
		delete m_Mesh;
	}
	if(m_MeshMaterials)
	{
		delete m_MeshMaterials;
	}
	if(m_MeshTextures && (*m_MeshTextures))
	{
		(*m_MeshTextures)->Release();
		delete m_MeshTextures;
	}

	

}

void RenderableObject::render(LPDIRECT3DDEVICE9 device)
{
	//LPDIRECT3DVERTEXBUFFER9 ppVB;
	//LPDIRECT3DINDEXBUFFER9 ppIB;
	//(*(*(this->m_Mesh)))->GetVertexBuffer(&ppVB);
	//(*(*(this->m_Mesh)))->GetIndexBuffer(&ppIB);
	//(*(*(this->m_Mesh)))->LockVertexBuffer( D3DLOCK_READONLY,(LPVOID*)&ppIB );
	//(*(*(this->m_Mesh)))->LockVertexBuffer( D3DLOCK_READONLY,(LPVOID*)&ppVB );
	render(device,worldM);
	//(*(*(this->m_Mesh)))->UnlockIndexBuffer();
	//(*(*(this->m_Mesh)))->UnlockVertexBuffer();
}

void RenderableObject::render(LPDIRECT3DDEVICE9 device,D3DXMATRIX &wM)
{
	device->SetTransform(D3DTS_WORLD,&wM);
	if( m_Mesh != NULL ){
		for (DWORD i = 0; i < m_NumMaterials; i++)
		{
		  // Set the material and texture for this subset
		 device->SetMaterial(&(m_MeshMaterials[i]));
		 device->SetTexture(0,m_MeshTextures[i]);

		(*m_Mesh)->DrawSubset(i);
		    
		}
	}

}
void RenderableObject::renderShadow(LPDIRECT3DDEVICE9 device,D3DXMATRIX &shadowM){
	D3DXMATRIX tWM =  worldM * shadowM;
	device->SetTransform(D3DTS_WORLD,&tWM);

	if( m_Mesh != NULL ){

		//this for loop may be seen unnecessary since shadow
		//drawing doesn't require meterial attachment
		//but meshes with mulitple material can be dividen 
		//into subsets, without drawing all the subsets
		//it is not possible to render all the shadow
		//of the object
		for (DWORD i = 0; i < m_NumMaterials; i++)
		{		    

		 // Draw the mesh subset
		 (*m_Mesh)->DrawSubset(i);
		}
	}
}



bool RenderableObject::LoadFromXFile(	LPCSTR filename, 
											LPDIRECT3DDEVICE9 device,																
											DWORD Options, 
											LPD3DXBUFFER *ppAdjacency,
											LPD3DXBUFFER *ppEffectInstances)
{
	LPD3DXMESH *l_mesh;
	DWORD l_numMaterials;
	LPD3DXBUFFER *l_materials;

	l_mesh = new LPD3DXMESH();
	l_numMaterials = 0;
	l_materials = new LPD3DXBUFFER();

	HRESULT hr = D3DXLoadMeshFromX (filename, Options,
               device, ppAdjacency,
               l_materials,ppEffectInstances, &l_numMaterials,
               l_mesh );

	if ( hr != S_OK )
		return false;


	D3DXMATERIAL *l_d3dxMaterials = (D3DXMATERIAL*)(*l_materials)->GetBufferPointer();
	D3DMATERIAL9 *l_meshMaterials = new D3DMATERIAL9[l_numMaterials];
	LPDIRECT3DTEXTURE9 *l_meshTextures = new LPDIRECT3DTEXTURE9[l_numMaterials];

	for (DWORD i=0; i<l_numMaterials; i++)
	{

		// Copy the material
		l_meshMaterials[i] = l_d3dxMaterials[i].MatD3D;

		// Set the ambient color for the material (D3DX does not do this)
		l_meshMaterials[i].Ambient = l_meshMaterials[i].Diffuse;

		// Create the texture if it exists - it may not
		l_meshTextures[i] = NULL;
		if (l_d3dxMaterials[i].pTextureFilename)
			D3DXCreateTextureFromFile(device, l_d3dxMaterials[i].pTextureFilename,   &l_meshTextures[i]); 

	}

	(this->m_Mesh) = l_mesh;
	(this->m_MeshMaterials) = l_meshMaterials;
	(this->m_MeshTextures) = l_meshTextures;
	this->m_NumMaterials = l_numMaterials;

	LPVOID _data = 0;

	D3DXVECTOR3	center(0,0,0);
	m_BoundingSphereRadius = 0;

	// compute radius
	if( (this->m_Mesh) )
	{
		HRESULT hr = (*m_Mesh)->LockVertexBuffer( D3DLOCK_READONLY, &_data );
		if( SUCCEEDED(hr) )
		{
			D3DXComputeBoundingSphere(
				(const D3DXVECTOR3 *)_data,
				(*m_Mesh)->GetNumVertices(),
				(*m_Mesh)->GetNumBytesPerVertex(),
				&center,
				&m_BoundingSphereRadius
			);


			(*m_Mesh)->UnlockVertexBuffer();
			m_BoundingSphereRadius /= 1.10f;
		}

	}


	m_CurrentState = RENDERABLE_LOADED;
	return true;
}






bool RenderableObject::LoadXFromMemory(	LPCVOID p_Memory, 
										DWORD p_SizeOfMem ,
										LPDIRECT3DDEVICE9 device,																
										DWORD Options, 
										LPD3DXBUFFER *ppAdjacency,
										LPD3DXBUFFER *ppEffectInstances)
{
	LPD3DXMESH *l_mesh;
	DWORD l_numMaterials;
	LPD3DXBUFFER *l_materials;

	l_mesh = new LPD3DXMESH();
	l_numMaterials = 0;
	l_materials = new LPD3DXBUFFER();

	HRESULT hr = D3DXLoadMeshFromXInMemory (p_Memory,p_SizeOfMem, Options,
               device, ppAdjacency,
               l_materials,ppEffectInstances, &l_numMaterials,
               l_mesh );

	if ( hr != S_OK )
		return false;


	D3DXMATERIAL *l_d3dxMaterials = (D3DXMATERIAL*)(*l_materials)->GetBufferPointer();
	D3DMATERIAL9 *l_meshMaterials = new D3DMATERIAL9[l_numMaterials];
	LPDIRECT3DTEXTURE9 *l_meshTextures = new LPDIRECT3DTEXTURE9[l_numMaterials];

	for (DWORD i=0; i<l_numMaterials; i++)
	{

		// Copy the material
		l_meshMaterials[i] = l_d3dxMaterials[i].MatD3D;

		// Set the ambient color for the material (D3DX does not do this)
		l_meshMaterials[i].Ambient = l_meshMaterials[i].Diffuse;

		// Create the texture if it exists - it may not
		l_meshTextures[i] = NULL;
		if (l_d3dxMaterials[i].pTextureFilename)
			D3DXCreateTextureFromFile(device, l_d3dxMaterials[i].pTextureFilename,   &l_meshTextures[i]); 

	}

	(this->m_Mesh) = l_mesh;
	(this->m_MeshMaterials) = l_meshMaterials;
	(this->m_MeshTextures) = l_meshTextures;
	this->m_NumMaterials = l_numMaterials;

	LPVOID _data = 0;

	D3DXVECTOR3	center(0,0,0);
	m_BoundingSphereRadius = 0;

	// compute radius
	if( (this->m_Mesh) )
	{
		HRESULT hr = (*(this->m_Mesh))->LockVertexBuffer( D3DLOCK_READONLY, &_data );
		if( SUCCEEDED(hr) )
		{
			D3DXComputeBoundingSphere(
				(const D3DXVECTOR3 *)_data,
				(*(this->m_Mesh))->GetNumVertices(),
				(*(this->m_Mesh))->GetNumBytesPerVertex(),
				&center,
				&m_BoundingSphereRadius
			);


			(*(this->m_Mesh))->UnlockVertexBuffer();
			m_BoundingSphereRadius /= 1.10f;
		}

	}


	m_CurrentState = RENDERABLE_LOADED;
	return true;
}



