#include "SceneManager.h"
#include "Stats.h"
#include "GameStats.h"

SceneManager::SceneManager(IDirect3DDevice9* &p_Device, WNDCLASS &p_Window, 
						   Mouse *p_Mouse, Keyboard *p_Keyboard)
{
	m_Device = p_Device;
	m_Window = p_Window;
	m_Mouse = p_Mouse;
	m_Keyboard = p_Keyboard;

	m_NOfListenerRuns = 0;
	m_PhysicsFrameListener = new PhysicsFrameListener();
	initRenderStates();
	initCamera();
	m_Sky = new Sky(m_Device,m_Camera,"grassenvmap1024.dds", 10000.0f);

	this->addListener(m_PhysicsFrameListener);
}

SceneManager::~SceneManager(void)
{
}

bool SceneManager::addListener(Listener *p_Listener)
{
	bool ret = false;
	if( p_Listener->isFrameListener() )
	{
		FrameListener *flTmp = (dynamic_cast<FrameListener*> (p_Listener)) ;
		flTmp->firstFrameStartedEventTime	= 
			flTmp->lastFrameStartedEventTime	= 
				flTmp->firstFrameEndedEventTime		=
					flTmp->lastFrameEndedEventTime		= (float) timeGetTime();

		m_FrameListeners.push_back( flTmp );
		ret = true;
	}

	if( p_Listener->isMouseListener() )
	{
		MouseListener *mlTmp = (dynamic_cast<MouseListener*> (p_Listener)) ;
		mlTmp->firstMouseEventTime = mlTmp->lastMouseEventTime = (float) timeGetTime();
		m_MouseListeners.push_back( mlTmp );
		ret = true;
	}

	if( p_Listener->isKeyboardListener() )
	{
		KeyboardListener *klTmp = (dynamic_cast<KeyboardListener*> (p_Listener)) ;
		m_KeyboardListeners.push_back( klTmp );
		ret = true;
	}


	return ret;
}


bool SceneManager::removeListener(Listener *p_Listener)
{
	return true;
}


void SceneManager::run( void )
{
	Stats::getSingleton()->updateNumOfFrame();
	runMouseListeners();
	runKeyboardListeners();
	runFrameListeners(true);
	
	renderLoop();


	runFrameListeners(false);
	m_NOfListenerRuns++;
}

void SceneManager::runFrameListeners(bool frameStarted)
{
	//vector<FrameListener*>::const_iterator iter = m_FrameListeners.begin();
	//vector<FrameListener*>::const_iterator endIter = m_FrameListeners.end();
	int i = 0;
	int sz = m_FrameListeners.size();
	FrameListener *tmpFL;
	float time;

	for( ; i < sz; i++ )
	{
		tmpFL = m_FrameListeners[i];
		if ( ( m_NOfListenerRuns + tmpFL->m_ID ) % tmpFL->m_Priority != 0 )
			continue;

		time = (float) timeGetTime();
		if( frameStarted )
		{
			tmpFL->frameStarted( time - tmpFL->lastFrameStartedEventTime );
			tmpFL->lastFrameStartedEventTime =  time;
		}
		else
		{
			tmpFL->frameEnded( time - tmpFL->lastFrameEndedEventTime );
			tmpFL->lastFrameEndedEventTime =  time;
		}
	}
	
}

void SceneManager::runMouseListeners()
{
	
	vector<MouseListener*>::const_iterator iter = m_MouseListeners.begin();
	vector<MouseListener*>::const_iterator endIter = m_MouseListeners.end();

	float time;
	

	// need to call update first since
	// isAnythingHappened is dependent from update
	if( m_Mouse->update() && m_Mouse->isAnythingHappened() )
	{
		for( ; iter != endIter ; iter++ )
		{


			if(m_Mouse->isAnyButtonDown())
			{
				time = (float) timeGetTime();
				(*iter)->mouseButtonPressed( m_Mouse->m_MouseButtonDownState,  time - (*iter)->lastMouseEventTime );
				(*iter)->lastMouseEventTime =  time;
			}
			if(m_Mouse->isAnyButtonReleased())
			{
				time = (float) timeGetTime();
				(*iter)->mouseButtonReleased( m_Mouse->m_MouseButtonReleaseState,  time - (*iter)->lastMouseEventTime );
				(*iter)->lastMouseEventTime =  time;
			}
			if(m_Mouse->isMouseMoved())
			{
				time = (float) timeGetTime();
				(*iter)->mouseMoved( m_Mouse->m_MouseButtonDownState,  time - (*iter)->lastMouseEventTime );
				(*iter)->lastMouseEventTime =  time;
			}
		}
	}
}

void SceneManager::runKeyboardListeners()
{
	vector<KeyboardListener*>::const_iterator iter = m_KeyboardListeners.begin();
	vector<KeyboardListener*>::const_iterator endIter = m_KeyboardListeners.end();

	float time;
	


	if( m_Keyboard->update() /*&&  m_Keyboard->isAnyKeyPressed() */)//isAnyKeyPressed deprecated
	{
		m_Keyboard->keyPressed(false);	//need to set false for the next test

		for( ; iter != endIter ; iter++ )
		{


			time = (float) timeGetTime();

			time = (float) timeGetTime();
			(*iter)->keyPressed( m_Keyboard,  time - (*iter)->lastKeyboardEventTime );
			(*iter)->lastKeyboardEventTime =  time;
		}
	}	

}

void	SceneManager::addRenderableObject (RenderableObject *p_Obj)
{
	if(p_Obj)
	{
		std::string tmpStr = p_Obj->m_Name;
		m_RenderableObjects.push_back(p_Obj);
//		m_RenderableObjectsMap.insert(tmpStr, p_Obj);

	}
}
void	SceneManager::removeRenderableObject (RenderableObject *p_Obj)
{

}
void	SceneManager::removeRenderableObject (string &p_Name)
{

}

void	SceneManager::initRenderStates ( void )
{
	//necessary renderstates for rendering the scene
	Tools::pushRenderState(m_Device, D3DRS_STENCILENABLE);
	Tools::pushRenderState(m_Device, D3DRS_CULLMODE);
	Tools::pushRenderState(m_Device, D3DRS_ALPHABLENDENABLE);
	Tools::pushRenderState(m_Device, D3DRS_ZENABLE);
	Tools::pushRenderState(m_Device, D3DRS_ZWRITEENABLE);
	
	m_Device->SetRenderState(D3DRS_STENCILENABLE, false);
	m_Device->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
	m_Device->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
	m_Device->SetRenderState(D3DRS_ZENABLE,true);
	m_Device->SetRenderState(D3DRS_ZWRITEENABLE, true);
}

void	SceneManager::initCamera ( void )
{
	m_Camera = new Camera();
	m_Camera->setLens(D3DX_PI * 0.25f, 4.0f/3.0f, 1.0f, 2000.0f);
	m_Camera->buildView();
	m_Device->SetTransform(D3DTS_PROJECTION, &(m_Camera->mViewProj));

}


void	SceneManager::renderLoop ( void )
{
	static DWORD prevTime = 0;
	static DWORD prevCleanTime = 0;

	PhysicsObject *phyO;
	RenderableObject *rendO;

	if(m_Device)
	{

		bool cleanFlag = false;
		if (timeGetTime() - prevCleanTime > 1000 )
		{
			cleanFlag = true;
		}

	
		m_Device->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL , D3DCOLOR_ARGB(0xff,0,0,0), 1.0f, 0 );


		// Begin the scene
		m_Device->BeginScene();
		{
			m_Sky->draw();

			m_Device->SetTransform(D3DTS_VIEW, &(m_Camera->mView));

			unsigned int i = 0;
			unsigned int szOfVector = m_RenderableObjects.size();

			if(cleanFlag){
				for( i = 0 ; i < szOfVector ; i++ )
				{
					if( m_Camera->mPosW.z - m_RenderableObjects[i]->getPosition().z > -40.0f)
					{
						m_RenderableObjects[i]->m_CurrentState = RENDERABLE_DEAD;
					}

					if(m_RenderableObjects[i]->m_CurrentState == RENDERABLE_DEAD)
					{
						rendO = m_RenderableObjects[i];
						phyO = dynamic_cast<PhysicsObject*> (rendO);
						if(phyO)
							PhysicsEngine::getSingleton()->unRegisterPhyObject( phyO );
						
						m_RenderableObjects.erase ( m_RenderableObjects.begin() + i);
						szOfVector = m_RenderableObjects.size();
						i--;
						delete rendO;
						rendO = NULL;
						continue;
					}
					//m_RenderableObjects[i]->render(m_Device);
				}
				prevCleanTime = timeGetTime();
				Stats::getSingleton()->m_NumOfRenderObjects = szOfVector;
			}
			szOfVector = m_RenderableObjects.size();
			for( i = 0; i < szOfVector ; i++ )
			{
				if(m_RenderableObjects[i] && m_RenderableObjects[i]->m_CurrentState != RENDERABLE_DEAD)
					m_RenderableObjects[i]->render(m_Device);
			}


			Stats::getSingleton()->updateRenderTime(timeGetTime() - prevTime);
			prevTime = timeGetTime();

			ParticleSystem::getSingleton()->render();

			szOfVector = m_Sprites.size();
			for( i = 0; i < szOfVector ; i++ )
			{
				if(m_Sprites[i])
					m_Sprites[i]->render(m_Device);
			}

		}


		//Stats::getSingleton()->display();
		GameStats::getSingleton()->display();

		m_Device->EndScene();
		m_Device->Present(0,0,0,0); 
	}
}