#pragma once
#include <vector>
#include <map>
#include "FrameListener.h"
#include "MouseListener.h"
#include "KeyboardListener.h"
#include "RenderableObject.h"
#include "Sky.h"
#include "Sprite.h"
#include "ParticleSystem.h"
#include "PhysicsFrameListener.h"

using namespace std;
using std::vector;
using std::map;
using std::string;


class SceneManager
{
public:

	vector	<MouseListener*>				m_MouseListeners;
	vector	<KeyboardListener*>				m_KeyboardListeners;
	vector	<FrameListener*>				m_FrameListeners;
	
	vector	<RenderableObject*>				m_RenderableObjects;
	vector	<Sprite*>						m_Sprites;

	std::map <std::string,RenderableObject*>m_RenderableObjectsMap;

	map		<std::string,Camera*>			m_Cameras;

	bool	addListener						(Listener *p_Listener);
	bool	removeListener					(Listener *p_Listener);

	void	run								( void );
	void	runFrameListeners				( bool frameStarted );
	void	runMouseListeners				( void );
	void	runKeyboardListeners			( void );

	void	addRenderableObject				(RenderableObject *p_Obj);
	void	removeRenderableObject			(RenderableObject *p_Obj);
	void	removeRenderableObject			(string &p_Name);

	void	initRenderStates				( void );
	void	initCamera						( void );

	void	renderLoop						( void );

	SceneManager							(IDirect3DDevice9* &p_Device, WNDCLASS &p_Window, 
															Mouse *p_Mouse, Keyboard *p_Keyboard);
	~SceneManager							( void );



	IDirect3DDevice9	*m_Device ;
	WNDCLASS			m_Window ;

	Keyboard			*m_Keyboard ;
	Mouse				*m_Mouse ;

	Camera				*m_Camera;
	Sky					*m_Sky;

	PhysicsFrameListener	*m_PhysicsFrameListener;
private:

	unsigned long int	m_NOfListenerRuns;
	unsigned long int	m_CleanCycle;

};
