//=============================================================================
// Sky.cpp by Frank Luna (C) 2004 All Rights Reserved.
//=============================================================================

#include "Sky.h"
#include "Tools.h"

Sky::Sky(LPDIRECT3DDEVICE9 p_Device, Camera *p_Camera,const std::string& envmapFilename, float skyRadius)
: mRadius(skyRadius)
{
	m_Device = p_Device;
	m_Camera = p_Camera;
	D3DXCreateSphere(m_Device, skyRadius, 30, 30, &mSphere, 0);
	D3DXCreateCubeTextureFromFile(m_Device, envmapFilename.c_str(), &mEnvMap);

	ID3DXBuffer* errors = 0;
	D3DXCreateEffectFromFileEx(m_Device, "sky.fx", 0, 0, 0, 
		0, 0, &mFX, &errors);
	if( errors )
		MessageBox(0, (char*)errors->GetBufferPointer(), 0, 0);

	mhTech   = mFX->GetTechniqueByName("SkyTech");
	mhWVP    = mFX->GetParameterByName(0, "gWVP");
	mhEnvMap = mFX->GetParameterByName(0, "gEnvMap");

	// Set effect parameters that do not vary.
	mFX->SetTechnique(mhTech);
	mFX->SetTexture(mhEnvMap, mEnvMap);
}

Sky::~Sky()
{
	if(mSphere)
		mSphere->Release();
	if(mEnvMap)
		mEnvMap->Release();
	if(mFX)
		mFX->Release();
}

DWORD Sky::getNumTriangles()
{
	return mSphere->GetNumFaces();
}

DWORD Sky::getNumVertices()
{
	return mSphere->GetNumVertices();
}

IDirect3DCubeTexture9* Sky::getEnvMap()
{
	return mEnvMap;
}

float Sky::getRadius()
{
	return mRadius;
}

void Sky::onLostDevice()
{
//	HR(mFX->OnLostDevice());
}

void Sky::onResetDevice()
{
//	HR(mFX->OnResetDevice());
}
	D3DXMATRIX rm;
float rot = 105.0f;
void Sky::draw()
{
	// Sky always centered about camera's position.
	D3DXMATRIX W;
	D3DXVECTOR3 p = m_Camera->pos();
	D3DXMatrixRotationX(&rm,GRAD2DEGREE(rot));
	D3DXMatrixTranslation(&W, p.x, p.y, p.z);
	W *= rm;
	mFX->SetMatrix(mhWVP, &(W*m_Camera->viewProj()));
	
	UINT numPasses = 0;
	mFX->Begin(&numPasses, 0);
	mFX->BeginPass(0);
	mSphere->DrawSubset(0);
	mFX->EndPass();
	mFX->End();
	rot+= .001f;
}