#include"Sound.h"
#include"Tools.h"

static	map<std::string,Sound*>		gs_SoundBank;

SAudioPathRef Sound::m_AudioPathRef[AUDIO_PATH_LIMIT];


Sound::Sound(char* szFile,IDirectMusicAudioPath8* pAudioPath)
{
	try
	{
		Audio *audio = Audio::getSingleton();
		WCHAR wszFile[MAX_PATH];

		m_Segment = NULL;
		m_Performance = audio->m_Performance;
		m_Loader = audio->m_Loader;

		//Create segment
		CoCreateInstance(CLSID_DirectMusicSegment,NULL,CLSCTX_INPROC,IID_IDirectMusicSegment8,(void**)&m_Segment);

		//Load segment from file
		MultiByteToWideChar(CP_ACP,0,szFile,-1,wszFile,MAX_PATH);

		m_Loader->LoadObjectFromFile(CLSID_DirectMusicSegment,IID_IDirectMusicSegment8,wszFile,(void**)&m_Segment);

		//Download band
		m_Segment->Download(m_Performance);

		//Init audio path and 3D sound buffer
		if(pAudioPath == NULL)
		{
			//Create 3D audio path
			m_Performance->CreateStandardAudioPath(DMUS_APATH_DYNAMIC_3D,64,true,&m_AudioPath);

			//Get 3D sound buffer from audio path
			m_AudioPath->GetObjectInPath(DMUS_PCHANNEL_ALL,DMUS_PATH_BUFFER,0,GUID_NULL,0,IID_IDirectSound3DBuffer,(void**)&m_3DBuffer);

			//Store audio path in reference counter
			for(unsigned i = 0;i < AUDIO_PATH_LIMIT;++i)
			{
				if(m_AudioPathRef[i].pAudioPath == NULL)
				{
					m_AudioPathRef[i].pAudioPath = m_AudioPath;
					m_AudioPathRef[i].p3DBuffer = m_3DBuffer;
					m_AudioPathRef[i].sRefCounter = 1;
					break;
				}
			}
		}
		else
		{
		//Copy audio path and increment reference counter
			for(unsigned i = 0;i < AUDIO_PATH_LIMIT;++i)
			{
				if(m_AudioPathRef[i].pAudioPath == pAudioPath)
				{
					m_AudioPath = m_AudioPathRef[i].pAudioPath;
					m_3DBuffer = m_AudioPathRef[i].p3DBuffer;
					m_AudioPathRef[i].sRefCounter++;
					break;
				}
			}
		}
	}
	catch (...)
	{
		Tools::LOG("Sound::Sound @ exception ",true);
		throw new exception();
	}
}//Sound


Sound::~Sound()
{
	//Release segment
	m_Segment->Unload(m_Performance);
	m_Segment->Release();

	//Decrement audio path reference counter and release audio path if reached zero 
	for(unsigned i = 0;i < AUDIO_PATH_LIMIT;++i)
	{
		if(m_AudioPathRef[i].pAudioPath == m_AudioPath)
		{
			//Decrement reference counter
			m_AudioPathRef[i].sRefCounter--;

			//Release audio path
			if(m_AudioPathRef[i].sRefCounter == 0)
			{
				m_3DBuffer->Release();
				m_AudioPath->Release();

				m_AudioPathRef[i].pAudioPath = NULL;
				m_AudioPathRef[i].p3DBuffer = NULL;
				m_AudioPathRef[i].sRefCounter = 0;
			}
			break;
		}
	}
}//~Sound


void Sound::play()
{
	m_Performance->PlaySegmentEx(m_Segment,0,NULL,DMUS_SEGF_SECONDARY,0,0,NULL,m_AudioPath);
}//Play

void Sound::stop()
{
	m_Performance->StopEx(m_Segment,0,0);
}//Stop


void Sound::activateAudioPath(bool bActivate)
{
	m_AudioPath->Activate(bActivate);
}//ActivateAudioPath


void Sound::setVolume(long lVolume,DWORD dwDuration)
{
	m_AudioPath->SetVolume(lVolume,dwDuration);
}//SetVolume

void Sound::setRepeats(DWORD dwRepeats)
{
	//A value of DMUS_SEG_REPEAT_INFINITE means infinite
	m_Segment->SetRepeats(dwRepeats);
}//SetRepeats


bool Sound::isPlaying()
{
	return (m_Performance->IsPlaying(m_Segment,NULL) == S_OK);
}//IsPlaying


Sound*	Sound::registerSound(char *p_Name, char *p_FileName)
{
	return registerSound(string(p_Name),string(p_FileName));
}

Sound*	Sound::registerSound(string &p_Name, string &p_FileName)
{
	try
	{
		if(gs_SoundBank[p_Name] == NULL)
		{
			gs_SoundBank[p_Name] = new Sound((char*)p_FileName.c_str(),NULL);
		}
		return gs_SoundBank[p_Name];
	}
	catch(...)
	{
		Tools::LOG("Sound::registerSound @ exception occured ");
		return NULL;
	}

}
Sound*	Sound::getSound(string &p_Name, bool p_NewInstance, string &p_FileName)
{
	Sound *ret = NULL ;
	try
	{
		if( !p_NewInstance )
		{
			ret = gs_SoundBank[p_Name] ;
			if ( ret == NULL )
				Tools::LOG("Sound::getSound :  "+p_Name+" not found");
		}
		else
		{
			ret = new Sound((char*)p_FileName.c_str(), NULL);
		}
	}
	catch(...)
	{
		Tools::LOG("Sound::getSound @ exception ");
	}


	return ret;

}
void	Sound::unregisterSound(string &p_Name)
{
	Sound *ret = gs_SoundBank[p_Name];
	if( ret == NULL)
	{
		Tools::LOG("Sound::unregisterSound :  "+p_Name+" not found");
	}

	else
	{
		gs_SoundBank.erase(p_Name);
	}



}
Sound* Sound::getSound(char *p_Name)
{
	return getSound(string(p_Name));
}

void	Sound::scanDirectory(string p_Dir)
{

	vector<string> files;
	Tools::searchDirectory(files,p_Dir,string("wav"),true);

	int i, sz;
	sz = files.size();

	Sound *tmp;
	string fullName;
	string name;
	for( i = 0 ; i < sz ; i++)
	{
		name = files[i].substr(0,files[i].size()-4);
		fullName = files[i];
		tmp = registerSound(name,fullName);
		if(tmp)
			Tools::LOG("Sound::scanDirectory @ " + fullName + " registered as: " + name);
		else
			Tools::LOG("Sound::scanDirectory @ " + fullName + " file cannot be registered ");

	}
}

