#include "SpaceShip.h"
#include "GameStats.h"

SpaceShip::SpaceShip(void)
:RenderableObject(SPACE_SHIP,std::string("SpaceShip")), PhysicsSphere(COLLISION_GROUP_A | COLLISION_GROUP_E)
{
	m_ZRotOrient = 0;
	m_XRotOrient = 0;

	m_Health = 100;
	m_Speed = 5.0f;
	m_IsTurnedBack = true;
	GameStats::getSingleton()->setHealth(m_Health);


	m_Lives = 5;
	PrimaryGunLayout *pgl	= new PrimaryGunLayout(11, &(this->worldM), 1300.0f,2.0f,D3DXCOLOR(.70f,0.1f,0.3f,0.9f));
	SecondGunLayout *sgl	= new SecondGunLayout(256, &(this->worldM), 4000.0f,2.0f,D3DXCOLOR(1.0f,1.0f,0.1f,1.0f));
	m_Gun[0] = new SpaceShipGun(&worldM, pgl,Sound::getSound("laser"),4.0f );
	m_Gun[1] = new SpaceShipGun(&worldM, sgl,Sound::getSound("gun2") ,0.1f );
	GameStats::getSingleton()->setLives(5);
}

SpaceShip::~SpaceShip(void)
{
}

void SpaceShip::reset()
{
	m_Lives--;

	m_Gun[0]->reset();
	m_Gun[1]->reset();
	m_Speed = 5.00f;
	m_Health = 100;

	teleport(D3DXVECTOR3( 0, 400.0f, 0 ) );
	m_ResetTime = timeGetTime();
	m_CollisionGroup = COLLISION_GROUP_NONE;

	GameStats::getSingleton()->setHealth(m_Health);
	GameStats::getSingleton()->setLives(m_Lives);


}
void	SpaceShip::turningBackToGame()
{
	if(worldM._42 > 4.0f && worldM._42 < -4.0 )
	{
		worldM._42 = 0;
	}
	else	
		worldM._42 /= -50.0f;
	
	if( timeGetTime() - m_ResetTime > 2000 )
	{
		worldM._42 = 0;
		m_IsTurnedBack = true;
		m_CollisionGroup = COLLISION_GROUP_A | COLLISION_GROUP_E;

	}





}

void	SpaceShip::render(LPDIRECT3DDEVICE9 p_Device)
{
	
	if( m_Health > 0 )
	{
		if(!m_IsTurnedBack)
			turningBackToGame();

		RenderableObject::render(p_Device);
	}
	else
	{
		reset();
		m_IsTurnedBack = false;
	}
}



D3DXVECTOR3	SpaceShip::getCenter()
{
	return getPosition();
}

float		SpaceShip::getRadius()
{
	return m_BoundingSphereRadius;
}


void	SpaceShip::rotateX(float p_Degree)
{

	float rotTmp;

	if( m_XRotOrient + p_Degree > 45.0f )
	{
		rotTmp = 45.0f - m_XRotOrient;
		m_XRotOrient = 45.0f;
	}
	else if (m_XRotOrient + p_Degree < -45.0f )
	{
		rotTmp = -45.0f - m_XRotOrient;
		m_XRotOrient = -45.0f;
	}
	else
	{
		rotTmp = p_Degree;
		m_XRotOrient += p_Degree;
	}
	Renderable::rotateX( rotTmp  );

}



void	SpaceShip::rotateZ(float p_Degree)
{
//	m_ZRotOrient += p_Degree;
	float rotTmp;

	if( m_ZRotOrient + p_Degree > 45.0f )
	{
		rotTmp = 45.0f - m_ZRotOrient;
		m_ZRotOrient = 45.0f;
	}
	else if (m_ZRotOrient + p_Degree < -45.0f )
	{
		rotTmp = -45.0f - m_ZRotOrient;
		m_ZRotOrient = -45.0f;
	}
	else
	{
		rotTmp = p_Degree;
		m_ZRotOrient += p_Degree;
	}
	Renderable::rotateZ( rotTmp  );
}


void	SpaceShip::rotateAbsoluteX(float p_Degree)
{
	rotateX(m_XRotOrient - p_Degree);
}


void	SpaceShip::rotateAbsoluteZ(float p_Degree)
{
	rotateZ(m_ZRotOrient - p_Degree);
}



void	SpaceShip::processCollision(PhysicsObject *p_PhyObject)
{
	if(	p_PhyObject->getPhysicsObjectType() == PhysicsObject::SPHERE )
	{
		SpaceShip* ship = dynamic_cast<SpaceShip*>(p_PhyObject);
	}
}