#include "Sprite.h"


SpriteLayout::SpriteLayout(string &p_TextureName, int p_TextureWidth, int p_TextureHeight, int p_NOfHorizontalFrames, int p_NOfVerticalFrames )

{
	m_Sprite = NULL;
	m_SpriteTexture = NULL;
	m_TextureName = p_TextureName;
	m_TextureWidth = p_TextureWidth; 
	m_TextureHeight = p_TextureHeight;
	m_NOfHorizontalFrames = p_NOfHorizontalFrames;
	m_NOfVerticalFrames = p_NOfVerticalFrames;

	m_NOfFrames = m_NOfHorizontalFrames * m_NOfVerticalFrames;
	m_FrameWidth = p_TextureWidth / m_NOfHorizontalFrames;
	m_FrameHeight = p_TextureHeight / m_NOfVerticalFrames;
}

SpriteLayout::~SpriteLayout()
{
	if(m_Sprite)
		m_Sprite->Release();
	if(m_SpriteTexture)
		m_SpriteTexture->Release();

	
	m_TextureName.clear();
}


void SpriteLayout::loadTexture( LPDIRECT3DDEVICE9	p_Device )
{
	D3DXIMAGE_INFO d3dxImageInfo;

	D3DXCreateTextureFromFileEx( p_Device,
								m_TextureName.c_str(),
								 m_TextureWidth, // I had to set width manually. D3DPOOL_DEFAULT works for textures but causes problems for D3DXSPRITE.
                                 m_TextureHeight, // I had to set height manually. D3DPOOL_DEFAULT works for textures but causes problems for D3DXSPRITE.
                                 1,   // Don't create mip-maps when you plan on using D3DXSPRITE. It throws off the pixel math for sprite animation.
                                 D3DPOOL_DEFAULT,
                                 D3DFMT_UNKNOWN,
                                 D3DPOOL_DEFAULT,
                                 D3DX_DEFAULT,
                                 D3DX_DEFAULT,
                                 D3DCOLOR_COLORVALUE(0.0f,0.0f,0.0f,1.0f),
                                 &d3dxImageInfo,
                                 NULL,
								 &m_SpriteTexture );

    p_Device->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	p_Device->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
}


void SpriteLayout::init(LPDIRECT3DDEVICE9	p_Device)
{
	loadTexture(p_Device);

	D3DXCreateSprite( p_Device, &m_Sprite );


}

Sprite::Sprite(SpriteLayout *p_SpriteLayout, float p_TimeBtwFrames,int p_NumOfLoops, DWORD p_RenderFlag , D3DXVECTOR3& p_RectOffset)
{
	m_SpriteLayout = p_SpriteLayout;
	m_CurrentFrame = 0;
	m_TimeBtwFrames = p_TimeBtwFrames;
	m_LastTime = 0;
	m_CurrentLoop = 1;
	m_NumOfLoops = p_NumOfLoops;
	m_Dead = false;
	m_RectOffset = p_RectOffset;
	m_RenderFlag = p_RenderFlag;
	refreshRectangle();

}

Sprite::~Sprite(void)
{
}

void	Sprite::refreshRectangle()
{
	m_SrcRect.top    = (( m_CurrentFrame % m_SpriteLayout->m_NOfVerticalFrames ) * m_SpriteLayout->m_FrameHeight);
	m_SrcRect.left   = (( m_CurrentFrame % m_SpriteLayout->m_NOfHorizontalFrames ) * m_SpriteLayout->m_FrameWidth);
	m_SrcRect.bottom = m_SrcRect.top  + m_SpriteLayout->m_FrameHeight;
	m_SrcRect.right  = m_SrcRect.left + m_SpriteLayout->m_FrameWidth;
}



void	Sprite::nextFrame(int p_NumOfFrame)
{
	m_CurrentFrame += p_NumOfFrame;
	//m_CurrentFrame %= m_SpriteLayout->m_NOfFrames;
	refreshRectangle();
}

void	Sprite::prevFrame(int p_NumOfFrame)
{
	p_NumOfFrame %= p_NumOfFrame;	
	m_CurrentFrame -= p_NumOfFrame;
	if(m_CurrentFrame < 0)
		m_CurrentFrame = m_SpriteLayout->m_NOfFrames;

	refreshRectangle();

}

void Sprite::render(LPDIRECT3DDEVICE9	p_Device)
{
	if ( /*m_TimeBtwFrames != 0 &&*/ m_TimeBtwFrames < timeGetTime() - m_LastTime )
	{
		m_LastTime = timeGetTime();
		nextFrame();
	}

	if(m_CurrentFrame >= m_SpriteLayout->m_NOfFrames)
	{
		m_CurrentLoop++;
	}

	if (m_NumOfLoops == 0 || m_CurrentLoop <= m_NumOfLoops)
	{
		m_CurrentFrame %= m_SpriteLayout->m_NOfFrames;
		// Build a source RECT which will copy only a small portion of the texture.


	 
		Tools::pushRenderState(p_Device, D3DRS_LIGHTING);
		Tools::pushRenderState(p_Device, D3DRS_CULLMODE);
		Tools::pushRenderState(p_Device, D3DRS_POINTSPRITEENABLE);
		Tools::pushRenderState(p_Device, D3DRS_POINTSCALEENABLE);



		p_Device->SetRenderState(D3DRS_LIGHTING, FALSE);
		p_Device->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		p_Device->SetRenderState(D3DRS_POINTSPRITEENABLE,true);
		p_Device->SetRenderState(D3DRS_POINTSCALEENABLE,true);

		
		m_SpriteLayout->m_Sprite->Begin( m_RenderFlag );


			m_SpriteLayout->m_Sprite->Draw( m_SpriteLayout->m_SpriteTexture,
								  &m_SrcRect,
								  &m_RectOffset,
								  NULL,
								  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f) );

		m_SpriteLayout->m_Sprite->End();


		Tools::popRenderState(p_Device);
		Tools::popRenderState(p_Device);
		Tools::popRenderState(p_Device);
		Tools::popRenderState(p_Device);

	}
	else
		m_Dead = true;



}
