

#include "Stats.h"
#include <tchar.h>
#include <string>
#include <sstream>
#include <iostream>
using std::ostringstream;

Stats::Stats(LPDIRECT3DDEVICE9 p_Device)
{
	D3DXFONT_DESC fontDesc;
	fontDesc.Height          = 24;
    fontDesc.Width           = 0;
    fontDesc.Weight          = 0;
    fontDesc.MipLevels       = 1;
    fontDesc.Italic          = false;
    fontDesc.CharSet         = DEFAULT_CHARSET;
    fontDesc.OutputPrecision = OUT_DEFAULT_PRECIS;
    fontDesc.Quality         = DEFAULT_QUALITY;
    fontDesc.PitchAndFamily  = DEFAULT_PITCH | FF_DONTCARE;
    _tcscpy(fontDesc.FaceName, _T("Times New Roman"));
	m_LastRun = 0;
	m_PhyTime = 0;
	m_RenderTime = 0;
	m_ParticleTime = 0;
	m_NumOfFrame = 0;
	m_NumOfRenderObjects = 0;
	m_NumOfParticles = 0;

	D3DXCreateFontIndirect(p_Device, &fontDesc, &m_Font);
}

Stats::~Stats()
{
	if(m_Font)
		m_Font->Release();
}



void Stats::display()
{
	static std::ostringstream ost;

	if( (float)timeGetTime() - m_LastRun > 300.0f )
	{


		float timeElapsed = (float)timeGetTime() - m_LastRun;
		m_LastRun = (float)timeGetTime();

		static float phyPercent, renderPercent, particlePercent, framesPerSec;

		float total = m_PhyTime + m_RenderTime + m_ParticleTime ;
		phyPercent = 100.0f * m_PhyTime / total;
		renderPercent = 100.0f * m_RenderTime / total;
		particlePercent = 100.0f * m_ParticleTime / total;
		framesPerSec = 1000.0f * m_NumOfFrame / timeElapsed;
		
		ost.str("");
		ost<<"in "<<timeElapsed<< " milliseconds";

		ost<<"\nFrames Per Second = " << framesPerSec ;
		ost<<"\nPhysics Time = " << m_PhyTime << "  "<< (int)phyPercent <<" %";
		ost<<"\n" << m_NumOfRenderObjects << " render objects";
		ost<<"\nRender Time = "<< m_RenderTime << "  "<<(int)renderPercent << " %";
		ost<< "\nParticle Time = " << m_ParticleTime <<"  " <<(int)particlePercent << " %" ;
		ost<< "\n"<<m_NumOfParticles<<" particles" ;
		;


		m_NumOfFrame = m_PhyTime = m_RenderTime = m_ParticleTime = 0.0f;
		

	}
	static RECT R = {5, 5, 0, 0};
	m_Font->DrawText(0, ost.str().c_str(), -1, &R, DT_NOCLIP, D3DCOLOR_XRGB(255,255,255));

	
}

Stats* g_StatsInstance = NULL;

void Stats::create(LPDIRECT3DDEVICE9 p_Device)
{
	if(g_StatsInstance == NULL)
		g_StatsInstance = new Stats(p_Device);
}

Stats* Stats::getSingleton()
{
	return g_StatsInstance;
}
