#include "Tools.h"
#include <stack>
#include <map>
using std::map;
LPDIRECT3DDEVICE9	g_Device;
typedef struct __RenderStatePair
{
	D3DRENDERSTATETYPE	state;
	DWORD				value;
}RenderStatePair;

using std::stack;
stack<RenderStatePair> g_RenderStateStack;

map<string, string>		g_GameVariables;


namespace Tools
{

	LPDIRECT3DDEVICE9	getDevice()
	{
		return g_Device;
	};
	void				setDevice(LPDIRECT3DDEVICE9 p_Device)
	{
		g_Device = p_Device;
	};

	ofstream logFile("Core-log.txt",ios::out);
	void LOG(string message, bool showOnScreen )
	{
		if(logFile)
		{
			logFile << ::timeGetTime() << "\t" <<  message.c_str() << endl;
		}
		if (showOnScreen)
			MessageBox( 0 , message.c_str(), 0, 0 );

	}

	void CloseLOGFile()
	{
		logFile.close();
	}

	ofstream* GetLogFile()
	{
		return &logFile;
	}

	float rnd( float fMin, float fMax )
	{
		float fRandNum = (float)( rand () ) / RAND_MAX;
		return fMin + (fMax - fMin) * fRandNum;
	}


	float fastSqrt( float number )
	{
		long i;
		float x2, y;
		const float threehalfs = 1.5f;

		x2 = number * 0.5f;
		y = number;
		i = * ( long * ) &y; // evil floating point bit level hacking
		i = 0x5f3759df - ( i >> 1 ); // what the fuck?
		y = * ( float * ) &i;
		y = y * ( threehalfs - ( x2 * y * y ) ); // 1st iteration
		y = y * ( threehalfs - ( x2 * y * y ) ); // 2nd iteration, this can be removed


		return 1.0f/y;
	}

	void pushRenderState( LPDIRECT3DDEVICE9 p_Device, D3DRENDERSTATETYPE p_State )
	{
		RenderStatePair p;

		p.state = p_State;
		p_Device->GetRenderState( p.state, &(p.value) );
		g_RenderStateStack.push(p);
	}
	void popRenderState( LPDIRECT3DDEVICE9 p_Device, int p_NumOfState)
	{
		try
		{
			int i ;
			for( i = 0 ; i < p_NumOfState ; i++)
			{
				RenderStatePair &p = g_RenderStateStack.top();
				p_Device->SetRenderState(p.state,p.value);
				g_RenderStateStack.pop();
			}
		}
		catch(...)
		{
			Tools::LOG("Tools::popRenderState @ exception ");
		}
	}



	int searchDirectory(std::vector<std::string> &refvecFiles,
						const std::string        &refcstrRootDirectory,
						const std::string        &refcstrExtension,
						bool                     bSearchSubdirectories)
	{
	  std::string     strFilePath;             // Filepath
	  std::string     strPattern;              // Pattern
	  std::string     strExtension;            // Extension
	  HANDLE          hFile;                   // Handle to file
	  WIN32_FIND_DATA FileInformation;         // File information


	  strPattern = refcstrRootDirectory + "\\*.*";

	  hFile = ::FindFirstFile(strPattern.c_str(), &FileInformation);
	  if(hFile != INVALID_HANDLE_VALUE)
	  {
		do
		{
		  if(FileInformation.cFileName[0] != '.')
		  {
			strFilePath.erase();
			strFilePath = /*refcstrRootDirectory + "\\"*/ + FileInformation.cFileName;

			if(FileInformation.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
			  if(bSearchSubdirectories)
			  {
				// Search subdirectory
				int iRC = searchDirectory(refvecFiles,
										  strFilePath,
										  refcstrExtension,
										  bSearchSubdirectories);
				if(iRC)
				  return iRC;
			  }
			}
			else
			{
			  // Check extension
			  strExtension = FileInformation.cFileName;
			  strExtension = strExtension.substr(strExtension.rfind(".") + 1);

			  if(strExtension == refcstrExtension)
			  {
				// Save filename
				  Tools::LOG("Tools::searchDirectory @ " +strFilePath+" is added to scanned files ");
				refvecFiles.push_back(strFilePath);
			  }
			}
		  }
		} while(::FindNextFile(hFile, &FileInformation) == TRUE);

		// Close handle
		::FindClose(hFile);

		DWORD dwError = ::GetLastError();
		if(dwError != ERROR_NO_MORE_FILES)
		  return dwError;
	  }

	  return 0;
	}


};



namespace GameVariables
{
	ifstream gameVariablesFile("Game-variables.txt");
	bool isGameVariablesRead = false;

	string&	getVariable(string &p_VariableName)
	{
		if( !isGameVariablesRead )
		{
			string s1;
			string s2;
			while (!gameVariablesFile.eof())
			{
				gameVariablesFile>>s1;
				gameVariablesFile>>s2;
				g_GameVariables[s1] = s2;
				Tools::LOG("GameVariables::getVariable @ variable name:'"+s1+"', value:'"+s2+"' recorded");
			}
			gameVariablesFile.close();
			isGameVariablesRead = true;

		}
		
		if ( g_GameVariables[p_VariableName] == string("") )
		{
				Tools::LOG("GameVariables::getVariable @ cannot find variable :'"+p_VariableName+"'");
		}

		return g_GameVariables[p_VariableName];
	}


	int		getInt(string &p_VariableName)
	{
		string& s = getVariable(p_VariableName);
		int ret;
		sscanf((const char*)s.c_str(),"%d",&ret);
		return ret;
	}

	float	getFloat(string &p_VariableName)
	{
		string& s = getVariable(p_VariableName);
		float ret;
		sscanf((const char*)s.c_str(),"%lf",&ret);
		return ret;

	}

};


